"""
.. _ref_operator_specification:

Operator Specification
======================
The OperatorSpecification Provides a documentation for each Operator
"""

import abc
from ansys.dpf.core import server as server_module
from ansys.dpf.gate import (
    operator_specification_capi,
    operator_specification_grpcapi,
    integral_types,
)
from ansys.dpf.core import mapping_types, common
from ansys.dpf.core.check_version import version_requires
from ansys.dpf.core.check_version import server_meet_version


class PinSpecification:
    """Documents an input or output pin of an Operator

    Parameters
    ----------
    name : str
        Name of the Pin.
    type_names : list[str], list[type], list[ansys.dpf.core.types], ansys.dpf.core.types, type, str
        List of accepted types.
    document : str, optional
        Explains what the pin is used for and what should be connect to it.
    optional : bool, optional
        Whether it is optional to connect to Pin or not. Default is False.
    ellipsis : bool, optional
        Whether data respecting this PinSpecification can be connected
        from this pin number to infinity. Default is False.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> operator = dpf.operators.result.displacement()
    >>> pin_spec = operator.specification.inputs[4]
    >>> pin_spec.name
    'data_sources'
    >>> pin_spec.type_names
    ['data_sources']
    >>> pin_spec.document
    'result file path container, used if no streams are set'
    >>> pin_spec.optional
    False
    """

    name: str
    _type_names: list
    document: str
    optional: bool
    ellipsis: bool
    name_derived_class = str

    def __init__(
        self, name, type_names, document="", optional=False, ellipsis=False, name_derived_class=""
    ):
        self.name = name
        self.type_names = type_names
        self.optional = optional
        self.document = document
        self.ellipsis = ellipsis
        self.name_derived_class = name_derived_class

    @property
    def type_names(self):
        """
        Returns
        -------
        list[str], list[type]
            List of accepted types.
        """
        return self._type_names

    @type_names.setter
    def type_names(self, val):
        if isinstance(val, str):
            self._type_names = [val]
            return
        elif isinstance(val, type):
            self._type_names = [mapping_types.map_types_to_cpp[val.__name__]]
            return
        elif isinstance(val, common.types):
            self._type_names = [
                mapping_types.map_types_to_cpp[common.types_enum_to_types()[val].__name__]
            ]
            return
        elif isinstance(val, list):
            if len(val) > 0 and isinstance(val[0], type):
                self._type_names = [mapping_types.map_types_to_cpp[ival.__name__] for ival in val]
                return
            if len(val) > 0 and isinstance(val[0], common.types):
                self._type_names = [
                    mapping_types.map_types_to_cpp[common.types_enum_to_types()[ival].__name__]
                    for ival in val
                ]
                return
        self._type_names = val

    @staticmethod
    def _get_copy(other, changed_types):
        return PinSpecification(
            other.name,
            changed_types,
            other.document,
            other.optional,
            other.ellipsis,
            other.name_derived_class,
        )

    def __repr__(self):
        return "{class_name}({params})".format(
            class_name=self.__class__.__name__,
            params=", ".join(
                "{param}={value}".format(param=k, value=f"'{v}'" if isinstance(v, str) else v)
                for k, v in vars(self).items()
                if not ("{param}" == "name_derived_class" and "name_derived_class" != "")
            ),
        )

    def __eq__(self, other):
        return str(self) == str(other)


class ConfigSpecification(dict):
    """Dictionary of the available configuration options and their specification
    (:class:`ansys.dpf.core.operator_specification.ConfigOptionSpec`)
    """

    def __init__(self, *arg, **kw):
        super(ConfigSpecification, self).__init__(*arg, **kw)


class ConfigOptionSpec:
    """Documentation of a configuration option available for a given
     Operator (:class:`ansys.dpf.core.Operator`)

    Attributes
    ----------
    name : str
        Name of the Configuration Option.
    type_names : list[str]
        List of accepted types.
    default_value_str : str
        Gives the stringified value of the default.
    document : str
        Documents the Configuration Option.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> operator = dpf.operators.math.add()
    >>> config_spec = operator.specification.config_specification
    >>> config_spec.keys()
    dict_keys(['binary_operation', 'inplace', 'mutex', 'num_threads', 'permissive', \
    'run_in_parallel', 'use_cache', 'work_by_index'])
    >>> config_spec['inplace']
    ConfigOptionSpec(name='inplace', type_names=['bool'], default_value_str='false', \
    document='The output is written over the input to save memory if this config is set to true.')

    """

    name: str
    type_names: list
    default_value_str: str
    document: str

    def __init__(self, name, type_names, default_value_str, document):
        self.name = name
        self.type_names = type_names
        self.default_value_str = default_value_str
        self.document = document

    def __repr__(self):
        return "{class_name}({params})".format(
            class_name=self.__class__.__name__,
            params=", ".join(
                "{param}={value}".format(param=k, value=f"'{v}'" if isinstance(v, str) else v)
                for k, v in vars(self).items()
            ),
        )


class SpecificationBase:
    @property
    @abc.abstractmethod
    def description(self):
        pass

    @property
    @abc.abstractmethod
    def inputs(self):
        pass

    @property
    @abc.abstractmethod
    def outputs(self):
        pass


class Specification(SpecificationBase):
    """Documents an Operator with its description (what the Operator does),
    its inputs and outputs and some properties.

    Examples
    --------
    Get the Specification of an operator by its name
    >>> from ansys.dpf import core as dpf
    >>> spec = dpf.operator_specification.Specification("U")
    >>> # or
    >>> spec = dpf.Operator.operator_specification("U")

    Get the specification of an instantiated operator
    >>> from ansys.dpf import core as dpf
    >>> operator = dpf.operators.result.displacement()
    >>> spec = operator.specification

    Display the Specification attributes
    >>> spec.description
    'Read/compute nodal displacements by calling the readers defined by the datasources.'
    >>> 4 in spec.inputs.keys()
    True
    >>> spec.outputs.keys()
    dict_keys([0])
    >>> spec.inputs[4].document
    'result file path container, used if no streams are set'
    """

    def __init__(self, operator_name=None, specification=None, server=None):
        # step 1: get server
        self._server = server_module.get_or_create_server(server)

        # step 2: get api
        self._api = self._server.get_api_for_type(
            capi=operator_specification_capi.OperatorSpecificationCAPI,
            grpcapi=operator_specification_grpcapi.OperatorSpecificationGRPCAPI,
        )

        # step3: init environment
        self._api.init_operator_specification_environment(self)  # creates stub when gRPC

        # step4: if object exists: take instance, else create it (specification)
        if specification is not None:
            self.internal_obj = specification
        else:
            if operator_name:
                if self._server.has_client():
                    self._internal_obj = self._api.operator_specification_new_on_client(
                        self._server.client, operator_name
                    )
                else:
                    self._internal_obj = self._api.operator_specification_new(operator_name)
            else:
                if self._server.has_client():
                    raise NotImplementedError(
                        "Creating an empty specification on a gRPC client is not implemented"
                    )
                self._internal_obj = self._api.operator_empty_specification_new()

        self.operator_name = operator_name
        self._map_output_pin_spec = None
        self._map_input_pin_spec = None
        self._properties = None
        self._config_specification = None

    def __str__(self):
        return "Description:\n" + str(self.description) + "\nProperties:\n" + str(self.properties)

    @property
    def properties(self):
        """Returns some additional properties of the Operator, like the category, the exposure,
        the scripting and user names and the plugin

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> operator = dpf.operators.math.add()
        >>> operator.specification.properties
        {'category': 'math', 'exposure': 'public', 'plugin': 'core', 'user_name': '+'}
        """
        if self._properties is None:
            temp_properties = dict()
            if self._internal_obj is not None:
                num_properties = self._api.operator_specification_get_num_properties(self)
                for i_property in range(num_properties):
                    property_key = self._api.operator_specification_get_property_key(
                        self, i_property
                    )
                    prop = self._api.operator_specification_get_properties(self, property_key)
                    temp_properties[property_key] = prop
            # Reorder the properties for consistency
            self._properties = dict()
            for key in sorted(temp_properties.keys()):
                self._properties[key] = temp_properties[key]
        return self._properties

    @property
    def description(self) -> str:
        """Returns a description of the operation applied by the Operator

        Returns
        -------
        str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> operator = dpf.operators.math.scale_by_field()
        >>> operator.specification.description
        "Scales a field (in 0) by a scalar field (in 1). If one field's ..."
        """
        if self._internal_obj is not None:
            return self._api.operator_specification_get_description(self)
        return ""

    @property
    def inputs(self) -> dict:
        """Returns a dictionary mapping the input pin numbers to their ``PinSpecification``

        Returns
        -------
        inputs : dict[int:PinSpecification]

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> operator = dpf.operators.mesh.mesh_provider()
        >>> 4 in operator.specification.inputs.keys()
        True
        >>> operator.specification.inputs[4]
        PinSpecification(name='data_sources', _type_names=['data_sources'], ...set', ellipsis=False,
         name_derived_class='')
        """
        if self._map_input_pin_spec is None:
            self._map_input_pin_spec = {}
            self._fill_pins(True, self._map_input_pin_spec)
        return self._map_input_pin_spec

    @property
    def outputs(self) -> dict:
        """Returns a dictionary mapping the output pin numbers to their ``PinSpecification``

        Returns
        -------
        outputs : dict[int:PinSpecification]

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> operator = dpf.operators.mesh.mesh_provider()
        >>> operator.specification.outputs
        {0: PinSpecification(name='mesh', _type_names=['abstract_meshed_region'], ...=False,
         name_derived_class='')}
        """
        if self._map_output_pin_spec is None:
            self._map_output_pin_spec = {}
            self._fill_pins(False, self._map_output_pin_spec)
        return self._map_output_pin_spec

    def _fill_pins(self, binput, to_fill):
        if self._internal_obj is not None:
            num_pins = self._api.operator_specification_get_num_pins(self, binput)

            pins = integral_types.MutableListInt32(size=num_pins)
            self._api.operator_specification_fill_pin_numbers(self, binput, pins)
            pins = pins.tolist()

            for i_pin in pins:
                pin_name = self._api.operator_specification_get_pin_name(self, binput, i_pin)
                pin_opt = self._api.operator_specification_is_pin_optional(self, binput, i_pin)
                pin_doc = self._api.operator_specification_get_pin_document(self, binput, i_pin)
                n_types = self._api.operator_specification_get_pin_num_type_names(
                    self, binput, i_pin
                )
                pin_type_names = [
                    self._api.operator_specification_get_pin_type_name(self, binput, i_pin, i_type)
                    for i_type in range(n_types)
                ]

                pin_derived_class_type_name = ""
                if server_meet_version("7.0", self._server) and hasattr(
                    self._api, "operator_specification_get_pin_derived_class_type_name"
                ):
                    pin_derived_class_type_name = (
                        self._api.operator_specification_get_pin_derived_class_type_name(
                            self, binput, i_pin
                        )
                    )

                pin_ell = self._api.operator_specification_is_pin_ellipsis(self, binput, i_pin)
                to_fill[i_pin] = PinSpecification(
                    pin_name,
                    pin_type_names,
                    pin_doc,
                    pin_opt,
                    pin_ell,
                    pin_derived_class_type_name,
                )

    @property
    def config_specification(self) -> ConfigSpecification:
        """Documents the available configuration options supported by the Operator

        Returns
        -------
        ConfigSpecification
        """
        if self._config_specification is None:
            self._config_specification = ConfigSpecification()
            num_options = self._api.operator_specification_get_num_config_options(self)
            for i in range(num_options):
                option_name = self._api.operator_specification_get_config_name(self, i)
                n_types = self._api.operator_specification_get_config_num_type_names(self, i)
                option_type_names = [
                    self._api.operator_specification_get_config_type_name(self, i, n_type)
                    for n_type in range(n_types)
                ]
                option_default_value = (
                    self._api.operator_specification_get_config_printable_default_value(self, i)
                )
                option_doc = self._api.operator_specification_get_config_description(self, i)
                self._config_specification[option_name] = ConfigOptionSpec(
                    name=option_name,
                    type_names=option_type_names,
                    default_value_str=option_default_value,
                    document=option_doc,
                )
        return self._config_specification


class CustomConfigOptionSpec(ConfigOptionSpec):
    def __init__(self, option_name: str, default_value, document: str):
        type_names = [mapping_types.map_types_to_cpp[type(default_value).__name__]]
        super().__init__(
            name=option_name,
            type_names=list(type_names),
            default_value_str=str(default_value),
            document=document,
        )


class Exposures:
    private = "private"
    public = "public"
    hidden = "hidden"


class Categories:
    result = "result"
    math = "math"
    mesh = "mesh"
    min_max = "min_max"
    scoping = "scoping"
    mapping = "mapping"
    geo = "geo"
    filter = "filter"
    utility = "utility"
    averaging = "averaging"
    serialization = "serialization"
    invariant = "invariant"
    logic = "logic"
    metadata = "metadata"


class SpecificationProperties:
    """Properties of an Operator.

    Parameters
    ----------
    user_name : str
        Readable lower case name of the Operator. example: "custom operator".

    category : str, Categories
        Choose from Categories options. Arrange the different Operators in the documentation
        and in the code generation.

    scripting_name : str
        Snake case name of the Operator. example: "custom_operator".

    exposure : Exposures
        Public by default, a hidden or private Operator doesn't appear in the documentation.

    plugin : str
        Snake case name of the plugin it belongs to.

    license: str
        Optional license name to check out that is used to run the operator.
        The value "any_dpf_supported_increments" tells DPF than any DPF-accepted license
        is accepted by this operator (see `here
        <https://dpf.docs.pyansys.com/version/stable/user_guide/getting_started_with_dpf_server.html#ansys-licensing>`_).  # noqa

    """

    def __init__(
        self,
        user_name: str = None,
        category: str = None,
        scripting_name: str = None,
        exposure: Exposures = Exposures.public,
        plugin: str = None,
        license: str = None,
        spec=None,
        **kwargs,
    ):
        if license is not None:
            kwargs["license"] = license

        self._spec = spec
        self.__dict__.update(
            user_name=user_name,
            category=category,
            exposure=exposure,
            scripting_name=scripting_name,
            plugin=plugin,
            **kwargs,
        )

    def __repr__(self):
        keys = sorted(self.__dict__)
        items = ("{}={!r}".format(k, self.__dict__[k]) for k in keys)
        return "{}({})".format(type(self).__name__, ", ".join(items))

    def __setitem__(self, key, value):
        if self._spec is not None:
            if value is not None:
                self._spec._api.operator_specification_set_property(self._spec, key, value)
                self._spec._properties = None
        setattr(self, key, value)

    def __getitem__(self, item: str):
        return getattr(self, item)

    def __eq__(self, other):
        return self.__dict__ == other.__dict__


class CustomSpecification(Specification):
    """Allows to create an Operator Specification with its description (what the Operator does),
    its inputs and outputs and some properties.
    Inherits from Specification (which has only getters) to implement setters.

    Designed to be used in an implementation of
    :class:`CustomOperatorBase <ansys.dpf.core.custom_operator.CustomOperatorBase>` for
    the property ``specification``.

    Notes
    -----
    Is only implemented for usage with type(server)=
    :class:`ansys.dpf.core.server_types.InProcessServer`
    and server version higher than 4.0.

    Examples
    --------
    >>> from ansys.dpf.core.custom_operator import CustomOperatorBase
    >>> from ansys.dpf.core import Field
    >>> from ansys.dpf.core.operator_specification import CustomSpecification, \
    SpecificationProperties, PinSpecification
    >>> class AddFloatToFieldData(CustomOperatorBase):
    ...     def run(self):
    ...         field = self.get_input(0, Field)
    ...         to_add = self.get_input(1, float)
    ...         data = field.data
    ...         data += to_add
    ...         self.set_output(0, field)
    ...         self.set_succeeded()
    ...
    ...     @property
    ...     def specification(self):
    ...         spec = CustomSpecification()
    ...         spec.description = "Add a custom value to all the data of an input Field"
    ...         spec.inputs = {
    ...             0: PinSpecification("field", [Field], "Field on which float value is added."),
    ...             1: PinSpecification("to_add", [float], "Data to add.") }
    ...         spec.outputs = {
    ...             0: PinSpecification("field", [Field], "Updated field.")}
    ...         spec.properties = SpecificationProperties("custom add to field", "math")
    ...         return spec
    ...
    ...     @property
    ...     def name(self):
    ...         return "custom_add_to_field"
    """

    def __init__(self, description=None, server=None):
        super().__init__(server=server)
        if description is not None:
            self.description = description

    @property
    @version_requires("4.0")
    def description(self) -> str:
        """Description of the operation applied by the Operator"""
        return super().description

    @description.setter
    def description(self, value) -> str:
        self._api.operator_specification_set_description(self, value)

    @property
    @version_requires("4.0")
    def inputs(self) -> dict:
        """Dictionary mapping the input pin numbers to their ``PinSpecification``

        Returns
        -------
        inputs : dict[int:PinSpecification]
        """
        return super().inputs

    @inputs.setter
    def inputs(self, val: dict):
        for key, value in val.items():
            list_types = integral_types.MutableListString(value.type_names)
            if server_meet_version("7.0", self._server) and hasattr(
                self._api, "operator_specification_set_pin_derived_class"
            ):
                self._api.operator_specification_set_pin_derived_class(
                    self,
                    True,
                    key,
                    value.name,
                    value.document,
                    len(value.type_names),
                    list_types,
                    value.optional,
                    value.ellipsis,
                    value.name_derived_class,
                )
            else:
                self._api.operator_specification_set_pin(
                    self,
                    True,
                    key,
                    value.name,
                    value.document,
                    len(value.type_names),
                    list_types,
                    value.optional,
                    value.ellipsis,
                )

    @property
    @version_requires("4.0")
    def outputs(self) -> dict:
        """Returns a dictionary mapping the output pin numbers to their ``PinSpecification``

        Returns
        -------
        outputs : dict[int:PinSpecification]
        """
        return super().outputs

    @outputs.setter
    def outputs(self, val: dict):
        for key, value in val.items():
            list_types = integral_types.MutableListString(value.type_names)
            if server_meet_version("7.0", self._server) and hasattr(
                self._api, "operator_specification_set_pin_derived_class"
            ):
                self._api.operator_specification_set_pin_derived_class(
                    self,
                    False,
                    key,
                    value.name,
                    value.document,
                    len(value.type_names),
                    list_types,
                    value.optional,
                    value.ellipsis,
                    value.name_derived_class,
                )
            else:
                self._api.operator_specification_set_pin(
                    self,
                    False,
                    key,
                    value.name,
                    value.document,
                    len(value.type_names),
                    list_types,
                    value.optional,
                    value.ellipsis,
                )

    @property
    @version_requires("4.0")
    def config_specification(self) -> ConfigSpecification:
        """Documents the available configuration options supported by the Operator

        Returns
        -------
        ConfigSpecification
        """
        return super().config_specification

    @config_specification.setter
    def config_specification(self, val: list):
        if isinstance(val, dict):
            val = [value for key, value in val.items()]
        for value in val:
            for type in value.type_names:
                if type == "double":
                    self._api.operator_specification_add_double_config_option(
                        self, value.name, float(value.default_value_str), value.document
                    )
                elif type == "int32":
                    self._api.operator_specification_add_int_config_option(
                        self,
                        value.name,
                        int(float(value.default_value_str)),
                        value.document,
                    )
                elif type == "bool":
                    self._api.operator_specification_add_bool_config_option(
                        self,
                        value.name,
                        value.default_value_str == "True",
                        value.document,
                    )
                else:
                    raise TypeError(
                        "config options are expected to be either boolean, integer or double values"
                    )

    @property
    @version_requires("4.0")
    def properties(self) -> SpecificationProperties:
        """Returns some additional properties of the Operator, like the category, the exposure,
        the scripting and user names and the plugin"""
        return SpecificationProperties(**super().properties, spec=self)

    @properties.setter
    def properties(self, val: SpecificationProperties):
        if isinstance(val, SpecificationProperties):
            val = val.__dict__
        for key, value in val.items():
            if value is not None:
                self._api.operator_specification_set_property(self, key, value)



"""
ServerContext
=============

Gives the ability to choose the context with which the server should be started.
The context allows you to choose the licensing logic for operators.
For every context, DPF always checks if an Ansys license is available.
By default, a **Premium** type of :class:`ServerContext` is used,
meaning that any operator requiring a license check-out can do so.
The **Entry** context instead does not allow operators to check a license out,
which will result in failure of operators requiring it.
The default context can be overwritten using the ANSYS_DPF_SERVER_CONTEXT environment
variable.
ANSYS_DPF_SERVER_CONTEXT=ENTRY and ANSYS_DPF_SERVER_CONTEXT=PREMIUM can be used.
"""
import os
import warnings
from enum import Enum
from ansys.dpf.core import dpf_operator
from ansys.dpf.core import errors


class LicensingContextType(Enum):
    premium = 1
    """Checks if at least one license increment exists
    and allows operators to block an increment."""
    entry = 4
    """Checks if at least one license increment exists
    and does not allow operators to block an increment."""

    def __int__(self):
        return self.value

    @staticmethod
    def same_licensing_context(first, second):
        if int(first) == int(LicensingContextType.entry) and int(second) != int(
            LicensingContextType.entry
        ):
            return False
        elif int(second) == int(LicensingContextType.entry) and int(first) != int(
            LicensingContextType.entry
        ):
            return False
        return True


class LicenseContextManager:
    """Can optionally be used to check out a license before using licensed DPF Operators.
    Improves performance if you are using multiple Operators that require licensing.
    It can also be used to force checkout before running a script when few
    Ansys license increments are available.
    The license is checked in when the object is deleted.

    Parameters
    ----------
    increment_name: str, optional
         License increment to check out. To improve script efficiency, this license increment
         should be consistent with the increments required by the following Operators. If ``None``,
         the first available increment of this
         `list <https://dpf.docs.pyansys.com/version/dev/user_guide/getting_started_with_dpf_server.
         html#ansys-licensing>`_
         is checked out.
    license_timeout_in_seconds: float, optional
         If an increment is not available by the maximum time set here, check out fails. Default is:
         :py:func:`ansys.dpf.core.runtime_config.RuntimeCoreConfig.license_timeout_in_seconds`
    server : server.DPFServer, optional
        Server with the channel connected to the remote or local instance. The
        default is ``None``, in which case an attempt is made to use the global
        server.

    Examples
    --------
    Using a context manager
    >>> from ansys.dpf import core as dpf
    >>> dpf.set_default_server_context(dpf.AvailableServerContexts.premium)
    >>> field = dpf.Field()
    >>> field.append([0., 0., 0.], 1)
    >>> op = dpf.operators.filter.field_high_pass()
    >>> op.inputs.field(field)
    >>> op.inputs.threshold(0.0)
    >>> with dpf.LicenseContextManager() as lic:
    ...    out = op.outputs.field()

    Using an instance
    >>> lic = dpf.LicenseContextManager()
    >>> op.inputs.field(field)
    >>> op.inputs.threshold(0.0)
    >>> out = op.outputs.field()
    >>> lic = None

    Using a context manager and choosing license options
    >>> op.inputs.field(field)
    >>> op.inputs.threshold(0.0)
    >>> out = op.outputs.field()
    >>> op = dpf.operators.filter.field_high_pass()
    >>> op.inputs.field(field)
    >>> op.inputs.threshold(0.0)
    >>> with dpf.LicenseContextManager(
    ...    increment_name="preppost", license_timeout_in_seconds=1.) as lic:
    ...    out = op.outputs.field()

    Notes
    -----
    Available from 6.1 server version.
    """

    def __init__(
        self, increment_name: str = None, license_timeout_in_seconds: float = None, server=None
    ):
        from ansys.dpf.core import server as server_module

        self._server = server_module.get_or_create_server(server)
        if not self._server.meet_version("6.1"):
            raise errors.DpfVersionNotSupported("6.1")
        self._license_checkout_operator = dpf_operator.Operator(
            "license_checkout", server=self._server
        )
        if increment_name is not None:
            self._license_checkout_operator.connect(0, increment_name)
        if license_timeout_in_seconds is not None:
            self._license_checkout_operator.connect(1, license_timeout_in_seconds)
        self._license_checkout_operator.run()

    def release_data(self):
        """Release the data."""
        self._license_checkout_operator = None

    def __enter__(self):
        return self

    def __exit__(self, type, value, tb):
        if tb is None:
            self.release_data()

    def __del__(self):
        self.release_data()
        pass

    @property
    def status(self):
        """Returns a string with the list of checked out increments.

        Returns
        -------
        str
        """
        status_operator = dpf_operator.Operator("license_status", server=self._server)
        return status_operator.eval()


class ServerContext:
    """The context defines whether DPF capabilities requiring a license checkout are allowed.
    xml_path argument won't be taken into account if using LicensingContextType.entry.

    Parameters
    ----------
    context_type : LicensingContextType
        Type of context.
    xml_path : str, optional
        Path to the xml to load.
    """

    def __init__(self, context_type=LicensingContextType.premium, xml_path=""):
        self._context_type = context_type
        self._xml_path = xml_path

    @property
    def licensing_context_type(self):
        """Whether capabilities requiring Licenses checkout should be allowed.

        Returns
        -------
        LicensingContextType
        """
        return self._context_type

    @property
    def xml_path(self):
        """Path to the xml listing the capabilities to load on the server.

        Returns
        -------
        str
        """
        return self._xml_path

    def __str__(self):
        return (
            f"Server Context of type {self.licensing_context_type}"
            f" with {'no' if len(self.xml_path) == 0 else ''} xml path"
            f"{'' if len(self.xml_path) == 0 else ': ' + self.xml_path}"
        )

    def __eq__(self, other):
        if not isinstance(other, ServerContext):
            return False
        return os.path.normpath(self.xml_path) == os.path.normpath(
            other.xml_path
        ) and LicensingContextType.same_licensing_context(
            self.licensing_context_type, other.licensing_context_type
        )

    def __ne__(self, other):
        return not self == other


class AvailableServerContexts:
    """Defines available server contexts."""

    pre_defined_environment = ServerContext(0)
    """DataProcessingCore.xml that is next to DataProcessingCore.dll/libDataProcessingCore.so will
    be taken"""
    premium = ServerContext(LicensingContextType.premium)
    """Gets the Specific premium DataProcessingCore.xml to load most plugins with their
    environments."""
    custom_defined = ServerContext(3)
    """Loads the xml named "DpfCustomDefined.xml" that the user can modify."""
    entry = ServerContext(LicensingContextType.entry)
    """Loads the minimum number of plugins for a basic usage. Is the default."""


DPF_SERVER_CONTEXT_ENV = "ANSYS_DPF_SERVER_CONTEXT"

SERVER_CONTEXT = AvailableServerContexts.premium
if DPF_SERVER_CONTEXT_ENV in os.environ.keys():
    default_context = os.getenv(DPF_SERVER_CONTEXT_ENV)
    try:
        SERVER_CONTEXT = getattr(AvailableServerContexts, default_context.lower())
    except AttributeError:
        warnings.warn(
            UserWarning(
                f"{DPF_SERVER_CONTEXT_ENV} is set to {default_context}, which is not "
                f"recognized as an available DPF ServerContext type. \n"
                f"Accepted values are: {[t.name.upper() for t in LicensingContextType]}.\n"
                f"Using {LicensingContextType.premium.name.upper()} "
                f"as the default ServerContext type."
            )
        )


def set_default_server_context(context=AvailableServerContexts.premium) -> None:
    """Sets this context as default for any new server. Also applies it to
    the global server if it is running as Entry and requested context is Premium.

    The context enables to choose whether DPF capabilities requiring a license checkout are allowed.

    Parameters
    ----------
    context : ServerContext
        Context to apply to the given server or to the newly started servers (when no server
        is given).

    Notes
    -----
    Available with server's version starting at 6.0 (Ansys 2023R2).
    """
    from ansys.dpf.core import SERVER

    global SERVER_CONTEXT
    SERVER_CONTEXT = context
    if SERVER is not None and context == AvailableServerContexts.premium:
        SERVER.apply_context(context)



"""
descending_sort_fc
==================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class descending_sort_fc(Operator):
    """Sort a field (in 0) in descending order, with an optional component
    priority table or a boolean to enable sort by scoping (in 1). This
    operator doesn't support multiple elementary data per entity.

    Parameters
    ----------
    fields_container : FieldsContainer
        Field or fields container with only one field
        is expected
    component_priority_table : optional
        Component priority table (vector of int)
    sort_by_scoping : bool, optional
        If true, uses scoping to sort the field
        (default is false)


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.logic.descending_sort_fc()

    >>> # Make input connections
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_component_priority_table = dpf.()
    >>> op.inputs.component_priority_table.connect(my_component_priority_table)
    >>> my_sort_by_scoping = bool()
    >>> op.inputs.sort_by_scoping.connect(my_sort_by_scoping)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.logic.descending_sort_fc(
    ...     fields_container=my_fields_container,
    ...     component_priority_table=my_component_priority_table,
    ...     sort_by_scoping=my_sort_by_scoping,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(
        self,
        fields_container=None,
        component_priority_table=None,
        sort_by_scoping=None,
        config=None,
        server=None,
    ):
        super().__init__(name="descending_sort_fc", config=config, server=server)
        self._inputs = InputsDescendingSortFc(self)
        self._outputs = OutputsDescendingSortFc(self)
        if fields_container is not None:
            self.inputs.fields_container.connect(fields_container)
        if component_priority_table is not None:
            self.inputs.component_priority_table.connect(component_priority_table)
        if sort_by_scoping is not None:
            self.inputs.sort_by_scoping.connect(sort_by_scoping)

    @staticmethod
    def _spec():
        description = """Sort a field (in 0) in descending order, with an optional component
            priority table or a boolean to enable sort by scoping (in
            1). This operator doesn't support multiple elementary data
            per entity."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Field or fields container with only one field
        is expected""",
                ),
                1: PinSpecification(
                    name="component_priority_table",
                    type_names=["vector<int32>"],
                    optional=True,
                    document="""Component priority table (vector of int)""",
                ),
                2: PinSpecification(
                    name="sort_by_scoping",
                    type_names=["bool"],
                    optional=True,
                    document="""If true, uses scoping to sort the field
        (default is false)""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="descending_sort_fc", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsDescendingSortFc
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsDescendingSortFc
        """
        return super().outputs


class InputsDescendingSortFc(_Inputs):
    """Intermediate class used to connect user inputs to
    descending_sort_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.logic.descending_sort_fc()
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_component_priority_table = dpf.()
    >>> op.inputs.component_priority_table.connect(my_component_priority_table)
    >>> my_sort_by_scoping = bool()
    >>> op.inputs.sort_by_scoping.connect(my_sort_by_scoping)
    """

    def __init__(self, op: Operator):
        super().__init__(descending_sort_fc._spec().inputs, op)
        self._fields_container = Input(
            descending_sort_fc._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._fields_container)
        self._component_priority_table = Input(
            descending_sort_fc._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._component_priority_table)
        self._sort_by_scoping = Input(
            descending_sort_fc._spec().input_pin(2), 2, op, -1
        )
        self._inputs.append(self._sort_by_scoping)

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator.

        Field or fields container with only one field
        is expected

        Parameters
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.descending_sort_fc()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> # or
        >>> op.inputs.fields_container(my_fields_container)
        """
        return self._fields_container

    @property
    def component_priority_table(self):
        """Allows to connect component_priority_table input to the operator.

        Component priority table (vector of int)

        Parameters
        ----------
        my_component_priority_table :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.descending_sort_fc()
        >>> op.inputs.component_priority_table.connect(my_component_priority_table)
        >>> # or
        >>> op.inputs.component_priority_table(my_component_priority_table)
        """
        return self._component_priority_table

    @property
    def sort_by_scoping(self):
        """Allows to connect sort_by_scoping input to the operator.

        If true, uses scoping to sort the field
        (default is false)

        Parameters
        ----------
        my_sort_by_scoping : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.descending_sort_fc()
        >>> op.inputs.sort_by_scoping.connect(my_sort_by_scoping)
        >>> # or
        >>> op.inputs.sort_by_scoping(my_sort_by_scoping)
        """
        return self._sort_by_scoping


class OutputsDescendingSortFc(_Outputs):
    """Intermediate class used to get outputs from
    descending_sort_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.logic.descending_sort_fc()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(descending_sort_fc._spec().outputs, op)
        self._fields_container = Output(descending_sort_fc._spec().output_pin(0), 0, op)
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator

        Returns
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.descending_sort_fc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """  # noqa: E501
        return self._fields_container



"""
on_mesh_property
================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class on_mesh_property(Operator):
    """Provides a scoping on a given property name and a property number.

    Parameters
    ----------
    requested_location : str, optional
        Nodal or elemental location are expected
    property_name : str
        Ex "apdl_element_type", "elprops", "mat",
        "eltype", "connectivity",
        "shell_elements", "solid_elements",
        "skin_elements", "beam_elements",
        "point_elements"...
    property_id : int, optional
    inclusive : int, optional
        Default is 1 (inclusive is true). only used
        if 'shape_values' property is
        requested. if inclusive is set to 1
        and 'elprops' property field is
        available, it will select all
        elements that are set on the
        corresponding property. if inclusive
        is set to 0 (exclusive) and 'elprops'
        property field is available, it will
        select the elements that are only set
        on this property.
    mesh : MeshedRegion


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.scoping.on_mesh_property()

    >>> # Make input connections
    >>> my_requested_location = str()
    >>> op.inputs.requested_location.connect(my_requested_location)
    >>> my_property_name = str()
    >>> op.inputs.property_name.connect(my_property_name)
    >>> my_property_id = int()
    >>> op.inputs.property_id.connect(my_property_id)
    >>> my_inclusive = int()
    >>> op.inputs.inclusive.connect(my_inclusive)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.scoping.on_mesh_property(
    ...     requested_location=my_requested_location,
    ...     property_name=my_property_name,
    ...     property_id=my_property_id,
    ...     inclusive=my_inclusive,
    ...     mesh=my_mesh,
    ... )

    >>> # Get output data
    >>> result_mesh_scoping = op.outputs.mesh_scoping()
    """

    def __init__(
        self,
        requested_location=None,
        property_name=None,
        property_id=None,
        inclusive=None,
        mesh=None,
        config=None,
        server=None,
    ):
        super().__init__(
            name="meshscoping_provider_by_prop", config=config, server=server
        )
        self._inputs = InputsOnMeshProperty(self)
        self._outputs = OutputsOnMeshProperty(self)
        if requested_location is not None:
            self.inputs.requested_location.connect(requested_location)
        if property_name is not None:
            self.inputs.property_name.connect(property_name)
        if property_id is not None:
            self.inputs.property_id.connect(property_id)
        if inclusive is not None:
            self.inputs.inclusive.connect(inclusive)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)

    @staticmethod
    def _spec():
        description = (
            """Provides a scoping on a given property name and a property number."""
        )
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="requested_location",
                    type_names=["string"],
                    optional=True,
                    document="""Nodal or elemental location are expected""",
                ),
                1: PinSpecification(
                    name="property_name",
                    type_names=["string"],
                    optional=False,
                    document="""Ex "apdl_element_type", "elprops", "mat",
        "eltype", "connectivity",
        "shell_elements", "solid_elements",
        "skin_elements", "beam_elements",
        "point_elements"...""",
                ),
                2: PinSpecification(
                    name="property_id",
                    type_names=["vector<int32>", "int32"],
                    optional=True,
                    document="""""",
                ),
                5: PinSpecification(
                    name="inclusive",
                    type_names=["int32"],
                    optional=True,
                    document="""Default is 1 (inclusive is true). only used
        if 'shape_values' property is
        requested. if inclusive is set to 1
        and 'elprops' property field is
        available, it will select all
        elements that are set on the
        corresponding property. if inclusive
        is set to 0 (exclusive) and 'elprops'
        property field is available, it will
        select the elements that are only set
        on this property.""",
                ),
                7: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="mesh_scoping",
                    type_names=["scoping"],
                    optional=False,
                    document="""Scoping""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(
            name="meshscoping_provider_by_prop", server=server
        )

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsOnMeshProperty
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsOnMeshProperty
        """
        return super().outputs


class InputsOnMeshProperty(_Inputs):
    """Intermediate class used to connect user inputs to
    on_mesh_property operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.scoping.on_mesh_property()
    >>> my_requested_location = str()
    >>> op.inputs.requested_location.connect(my_requested_location)
    >>> my_property_name = str()
    >>> op.inputs.property_name.connect(my_property_name)
    >>> my_property_id = int()
    >>> op.inputs.property_id.connect(my_property_id)
    >>> my_inclusive = int()
    >>> op.inputs.inclusive.connect(my_inclusive)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    """

    def __init__(self, op: Operator):
        super().__init__(on_mesh_property._spec().inputs, op)
        self._requested_location = Input(
            on_mesh_property._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._requested_location)
        self._property_name = Input(on_mesh_property._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._property_name)
        self._property_id = Input(on_mesh_property._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._property_id)
        self._inclusive = Input(on_mesh_property._spec().input_pin(5), 5, op, -1)
        self._inputs.append(self._inclusive)
        self._mesh = Input(on_mesh_property._spec().input_pin(7), 7, op, -1)
        self._inputs.append(self._mesh)

    @property
    def requested_location(self):
        """Allows to connect requested_location input to the operator.

        Nodal or elemental location are expected

        Parameters
        ----------
        my_requested_location : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.on_mesh_property()
        >>> op.inputs.requested_location.connect(my_requested_location)
        >>> # or
        >>> op.inputs.requested_location(my_requested_location)
        """
        return self._requested_location

    @property
    def property_name(self):
        """Allows to connect property_name input to the operator.

        Ex "apdl_element_type", "elprops", "mat",
        "eltype", "connectivity",
        "shell_elements", "solid_elements",
        "skin_elements", "beam_elements",
        "point_elements"...

        Parameters
        ----------
        my_property_name : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.on_mesh_property()
        >>> op.inputs.property_name.connect(my_property_name)
        >>> # or
        >>> op.inputs.property_name(my_property_name)
        """
        return self._property_name

    @property
    def property_id(self):
        """Allows to connect property_id input to the operator.

        Parameters
        ----------
        my_property_id : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.on_mesh_property()
        >>> op.inputs.property_id.connect(my_property_id)
        >>> # or
        >>> op.inputs.property_id(my_property_id)
        """
        return self._property_id

    @property
    def inclusive(self):
        """Allows to connect inclusive input to the operator.

        Default is 1 (inclusive is true). only used
        if 'shape_values' property is
        requested. if inclusive is set to 1
        and 'elprops' property field is
        available, it will select all
        elements that are set on the
        corresponding property. if inclusive
        is set to 0 (exclusive) and 'elprops'
        property field is available, it will
        select the elements that are only set
        on this property.

        Parameters
        ----------
        my_inclusive : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.on_mesh_property()
        >>> op.inputs.inclusive.connect(my_inclusive)
        >>> # or
        >>> op.inputs.inclusive(my_inclusive)
        """
        return self._inclusive

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.on_mesh_property()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh


class OutputsOnMeshProperty(_Outputs):
    """Intermediate class used to get outputs from
    on_mesh_property operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.scoping.on_mesh_property()
    >>> # Connect inputs : op.inputs. ...
    >>> result_mesh_scoping = op.outputs.mesh_scoping()
    """

    def __init__(self, op: Operator):
        super().__init__(on_mesh_property._spec().outputs, op)
        self._mesh_scoping = Output(on_mesh_property._spec().output_pin(0), 0, op)
        self._outputs.append(self._mesh_scoping)

    @property
    def mesh_scoping(self):
        """Allows to get mesh_scoping output of the operator

        Returns
        ----------
        my_mesh_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.on_mesh_property()
        >>> # Connect inputs : op.inputs. ...
        >>> result_mesh_scoping = op.outputs.mesh_scoping()
        """  # noqa: E501
        return self._mesh_scoping



"""
merge_generic_data_container
============================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class merge_generic_data_container(Operator):
    """Merges a list of generic data container. For each data entry, the
    merge operation is forwarded to the correct merge Operator.
    Primitive types cannot be merged, first instance found will be
    maintained in the result.

    Parameters
    ----------
    generic_data_container1 : GenericDataContainer
        Either a vector of generic data containers
        (sharing the same data types) or
        generic data containers from pin 0 to
        ... to merge. supported types rely on
        existing type specific merge
        operators.
    generic_data_container2 : GenericDataContainer
        Either a vector of generic data containers
        (sharing the same data types) or
        generic data containers from pin 0 to
        ... to merge. supported types rely on
        existing type specific merge
        operators.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.merge_generic_data_container()

    >>> # Make input connections
    >>> my_generic_data_container1 = dpf.GenericDataContainer()
    >>> op.inputs.generic_data_container1.connect(my_generic_data_container1)
    >>> my_generic_data_container2 = dpf.GenericDataContainer()
    >>> op.inputs.generic_data_container2.connect(my_generic_data_container2)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.merge_generic_data_container(
    ...     generic_data_container1=my_generic_data_container1,
    ...     generic_data_container2=my_generic_data_container2,
    ... )

    >>> # Get output data
    >>> result_any = op.outputs.any()
    """

    def __init__(
        self,
        generic_data_container1=None,
        generic_data_container2=None,
        config=None,
        server=None,
    ):
        super().__init__(
            name="merge::generic_data_container", config=config, server=server
        )
        self._inputs = InputsMergeGenericDataContainer(self)
        self._outputs = OutputsMergeGenericDataContainer(self)
        if generic_data_container1 is not None:
            self.inputs.generic_data_container1.connect(generic_data_container1)
        if generic_data_container2 is not None:
            self.inputs.generic_data_container2.connect(generic_data_container2)

    @staticmethod
    def _spec():
        description = """Merges a list of generic data container. For each data entry, the
            merge operation is forwarded to the correct merge
            Operator. Primitive types cannot be merged, first instance
            found will be maintained in the result."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="generic_data_container",
                    type_names=["generic_data_container"],
                    optional=False,
                    document="""Either a vector of generic data containers
        (sharing the same data types) or
        generic data containers from pin 0 to
        ... to merge. supported types rely on
        existing type specific merge
        operators.""",
                ),
                1: PinSpecification(
                    name="generic_data_container",
                    type_names=["generic_data_container"],
                    optional=False,
                    document="""Either a vector of generic data containers
        (sharing the same data types) or
        generic data containers from pin 0 to
        ... to merge. supported types rely on
        existing type specific merge
        operators.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="any",
                    type_names=["any"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(
            name="merge::generic_data_container", server=server
        )

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMergeGenericDataContainer
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMergeGenericDataContainer
        """
        return super().outputs


class InputsMergeGenericDataContainer(_Inputs):
    """Intermediate class used to connect user inputs to
    merge_generic_data_container operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.merge_generic_data_container()
    >>> my_generic_data_container1 = dpf.GenericDataContainer()
    >>> op.inputs.generic_data_container1.connect(my_generic_data_container1)
    >>> my_generic_data_container2 = dpf.GenericDataContainer()
    >>> op.inputs.generic_data_container2.connect(my_generic_data_container2)
    """

    def __init__(self, op: Operator):
        super().__init__(merge_generic_data_container._spec().inputs, op)
        self._generic_data_container1 = Input(
            merge_generic_data_container._spec().input_pin(0), 0, op, 0
        )
        self._inputs.append(self._generic_data_container1)
        self._generic_data_container2 = Input(
            merge_generic_data_container._spec().input_pin(1), 1, op, 1
        )
        self._inputs.append(self._generic_data_container2)

    @property
    def generic_data_container1(self):
        """Allows to connect generic_data_container1 input to the operator.

        Either a vector of generic data containers
        (sharing the same data types) or
        generic data containers from pin 0 to
        ... to merge. supported types rely on
        existing type specific merge
        operators.

        Parameters
        ----------
        my_generic_data_container1 : GenericDataContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_generic_data_container()
        >>> op.inputs.generic_data_container1.connect(my_generic_data_container1)
        >>> # or
        >>> op.inputs.generic_data_container1(my_generic_data_container1)
        """
        return self._generic_data_container1

    @property
    def generic_data_container2(self):
        """Allows to connect generic_data_container2 input to the operator.

        Either a vector of generic data containers
        (sharing the same data types) or
        generic data containers from pin 0 to
        ... to merge. supported types rely on
        existing type specific merge
        operators.

        Parameters
        ----------
        my_generic_data_container2 : GenericDataContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_generic_data_container()
        >>> op.inputs.generic_data_container2.connect(my_generic_data_container2)
        >>> # or
        >>> op.inputs.generic_data_container2(my_generic_data_container2)
        """
        return self._generic_data_container2


class OutputsMergeGenericDataContainer(_Outputs):
    """Intermediate class used to get outputs from
    merge_generic_data_container operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.merge_generic_data_container()
    >>> # Connect inputs : op.inputs. ...
    >>> result_any = op.outputs.any()
    """

    def __init__(self, op: Operator):
        super().__init__(merge_generic_data_container._spec().outputs, op)
        self._any = Output(merge_generic_data_container._spec().output_pin(0), 0, op)
        self._outputs.append(self._any)

    @property
    def any(self):
        """Allows to get any output of the operator

        Returns
        ----------
        my_any : Any

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_generic_data_container()
        >>> # Connect inputs : op.inputs. ...
        >>> result_any = op.outputs.any()
        """  # noqa: E501
        return self._any



"""
add_fc
======
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class add_fc(Operator):
    """Selects all fields with the same label space in the input fields
    container and add those together. If fields, doubles, or vectors
    of doubles, are put in input they are added to all the fields.

    Parameters
    ----------
    fields_container1 : FieldsContainer or Field or float
    fields_container2 : FieldsContainer or Field or float


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.math.add_fc()

    >>> # Make input connections
    >>> my_fields_container1 = dpf.FieldsContainer()
    >>> op.inputs.fields_container1.connect(my_fields_container1)
    >>> my_fields_container2 = dpf.FieldsContainer()
    >>> op.inputs.fields_container2.connect(my_fields_container2)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.math.add_fc(
    ...     fields_container1=my_fields_container1,
    ...     fields_container2=my_fields_container2,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(
        self, fields_container1=None, fields_container2=None, config=None, server=None
    ):
        super().__init__(name="add_fc", config=config, server=server)
        self._inputs = InputsAddFc(self)
        self._outputs = OutputsAddFc(self)
        if fields_container1 is not None:
            self.inputs.fields_container1.connect(fields_container1)
        if fields_container2 is not None:
            self.inputs.fields_container2.connect(fields_container2)

    @staticmethod
    def _spec():
        description = """Selects all fields with the same label space in the input fields
            container and add those together. If fields, doubles, or
            vectors of doubles, are put in input they are added to all
            the fields."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=[
                        "fields_container",
                        "field",
                        "double",
                        "vector<double>",
                    ],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="fields_container",
                    type_names=[
                        "fields_container",
                        "field",
                        "double",
                        "vector<double>",
                    ],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="add_fc", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsAddFc
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsAddFc
        """
        return super().outputs


class InputsAddFc(_Inputs):
    """Intermediate class used to connect user inputs to
    add_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.math.add_fc()
    >>> my_fields_container1 = dpf.FieldsContainer()
    >>> op.inputs.fields_container1.connect(my_fields_container1)
    >>> my_fields_container2 = dpf.FieldsContainer()
    >>> op.inputs.fields_container2.connect(my_fields_container2)
    """

    def __init__(self, op: Operator):
        super().__init__(add_fc._spec().inputs, op)
        self._fields_container1 = Input(add_fc._spec().input_pin(0), 0, op, 0)
        self._inputs.append(self._fields_container1)
        self._fields_container2 = Input(add_fc._spec().input_pin(1), 1, op, 1)
        self._inputs.append(self._fields_container2)

    @property
    def fields_container1(self):
        """Allows to connect fields_container1 input to the operator.

        Parameters
        ----------
        my_fields_container1 : FieldsContainer or Field or float

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.add_fc()
        >>> op.inputs.fields_container1.connect(my_fields_container1)
        >>> # or
        >>> op.inputs.fields_container1(my_fields_container1)
        """
        return self._fields_container1

    @property
    def fields_container2(self):
        """Allows to connect fields_container2 input to the operator.

        Parameters
        ----------
        my_fields_container2 : FieldsContainer or Field or float

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.add_fc()
        >>> op.inputs.fields_container2.connect(my_fields_container2)
        >>> # or
        >>> op.inputs.fields_container2(my_fields_container2)
        """
        return self._fields_container2


class OutputsAddFc(_Outputs):
    """Intermediate class used to get outputs from
    add_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.math.add_fc()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(add_fc._spec().outputs, op)
        self._fields_container = Output(add_fc._spec().output_pin(0), 0, op)
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator

        Returns
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.add_fc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """  # noqa: E501
        return self._fields_container



"""
.. _ref_property_fields_container:

_MockPropertyFieldsContainer
============================
Contains classes associated with the _MockPropertyFieldsContainer.
"""
from __future__ import annotations

from collections.abc import Sequence
import copy
from typing import Dict, List, Union

import numpy as np

import ansys.dpf.core as dpf
from ansys.dpf.core import PropertyField
from ansys.dpf.core.server_types import BaseServer


class _LabelSpaceKV:
    """Class for internal use to associate a label space with a field."""

    def __init__(self, _dict: Dict[str, int], _field: dpf.Field):
        """Constructs an association between a dictionary and a field."""
        self._dict = _dict
        self._field = _field

    @property
    def dict(self) -> dict:
        """Returns the associated dictionary."""
        return self._dict

    @property
    def field(self) -> dpf.Field:
        """Returns the associated field."""
        return self._field

    @field.setter
    def field(self, value: dpf.Field):
        self._field = value

    def __str__(self):
        """Returns a string representation of the association."""
        field_str = str(self._field).replace("\n", "\n\t\t\t")
        return f"Label Space: {self._dict} with field\n\t\t\t{field_str}"


class _MockPropertyFieldsContainer(Sequence):
    """Minimal implementation of a FieldsContainer specialized for _MockPropertyFieldsContainer."""

    def __init__(
        self,
        fields_container: _MockPropertyFieldsContainer = None,
        server: BaseServer = None,
    ):
        """Constructs a _MockPropertyFieldsContainer."""
        # default constructor
        self._labels = []  # used by Dataframe
        self.scopings = []
        self._server = None  # used by Dataframe

        self.label_spaces = []
        self.ids = []

        # _MockPropertyFieldsContainer copy
        if fields_container is not None:
            self._labels = copy.deepcopy(fields_container.labels)
            # self.scopings = copy.deepcopy(fields_container.scopings)
            self._server = fields_container._server

            # self.ids = copy.deepcopy(fields_container.ids)

            for ls in fields_container.label_spaces:
                self.add_entry(copy.deepcopy(ls.dict), ls.field.as_local_field())

        # server copy
        if server is not None:
            self._server = server

    # Collection
    def __str__(self) -> str:
        """Returns a string representation of a _MockPropertyFieldsContainer."""
        txt = f"DPF PropertyFieldsContainer with {len(self)} fields\n"
        for idx, ls in enumerate(self.label_spaces):
            txt += f"\t {idx}: {ls}\n"

        return txt

    @property
    def labels(self) -> List[str]:
        """Returns all labels of the _MockPropertyFieldsContainer."""
        return self._labels

    @labels.setter
    def labels(self, labels: List[str]):
        """Sets all the label of the _MockPropertyFieldsContainer."""
        if len(self._labels) != 0:
            raise ValueError("labels already set")
        for l in labels:
            self.add_label(l)

    def add_label(self, label: str):
        """Adds a label."""
        if label not in self._labels:
            self._labels.append(label)
            self.scopings.append([])

    def has_label(self, label) -> bool:
        """Check if a _MockPropertyFieldsContainer contains a given label."""
        return label in self.labels

    # used by Dataframe
    def get_label_space(self, idx) -> Dict:
        """Get a Label Space at a given index."""
        return self.label_spaces[idx].dict

    # used by Dataframe
    def get_label_scoping(self, label="time") -> dpf.Scoping:
        """Returns a scoping on the fields concerned by the given label."""
        if label in self.labels:
            scoping_ids = self.scopings[self.labels.index(label)]
            return dpf.Scoping(ids=scoping_ids, location="")
        raise KeyError(f"label {label} not found")

    def add_entry(self, label_space: Dict[str, int], value: dpf.Field):
        """Adds a PropertyField associated with a dictionary."""
        new_id = self._new_id()

        if hasattr(value, "_server"):
            self._server = value._server

        # add Label Space
        self.label_spaces.append(_LabelSpaceKV(label_space, value))

        # Update IDs
        self.ids.append(new_id)

        # Update Scopings
        for label in label_space.keys():
            label_idx = self.labels.index(label)
            self.scopings[label_idx].append(new_id)

    def add_field(self, label_space: Dict[str, int], field: dpf.Field):
        """Add or update a field at a requested label space."""
        self.add_entry(label_space, field)

    def get_entries(self, label_space_or_index: Union[Dict[str, int], int]):
        """Returns a list of fields from a complete or partial specification of a dictionary."""
        if isinstance(label_space_or_index, int):
            idx: int = label_space_or_index
            return [self.label_spaces[idx].field]
        else:
            _dict: Dict[str, int] = label_space_or_index
            are_keys_in_labels = [key in self.labels for key in _dict.keys()]
            if all(are_keys_in_labels):
                remaining = set(range(len(self.label_spaces)))
                for key in _dict.keys():
                    val = _dict[key]
                    to_remove = set()
                    for idx in remaining:
                        ls = self.label_spaces[idx]
                        if key in ls.dict.keys():
                            if ls.dict[key] != val:
                                to_remove.add(idx)
                        else:
                            to_remove.add(idx)
                    remaining = remaining.difference(to_remove)

                idx_to_field = lambda idx: self.label_spaces[idx].field
                return list(map(idx_to_field, remaining))
            else:
                bad_idx = are_keys_in_labels.index(False)
                bad_key = list(_dict.keys())[bad_idx]
                raise KeyError(f"Key {bad_key} is not in labels: {self.labels}")

    def get_entry(self, label_space_or_index: Union[Dict[str, int], int]):
        """Returns the field or (first field found) corresponding to the given dictionary."""
        ret = self.get_entries(label_space_or_index)

        if len(ret) != 0:
            return ret[0]

        raise ValueError("Could not find corresponding entry")

    def _new_id(self) -> int:
        """Helper method generating a new id when calling add_entry(...)."""
        if len(self.ids) == 0:
            self.last_id = 1
            return self.last_id
        else:
            self.last_id += 1
            return self.last_id

    # used by Dataframe
    def get_fields(self, label_space: Dict[str, int]) -> List[dpf.Field]:
        """Returns the list of fields associated with given label space."""
        return self.get_entries(label_space)

    def get_field(self, label_space_or_index: Union[Dict[str, int], int]) -> dpf.Field:
        """Retrieves the field at a requested index or label space."""
        return self.get_entry(label_space_or_index)

    # used by Dataframe
    def __getitem__(self, key: Union[Dict[str, int], int]) -> dpf.Field:
        """Retrieve the field at a requested index."""
        return self.get_field(key)

    def __len__(self) -> int:
        """Retrieve the number of label spaces."""
        return len(self.label_spaces)

    def _set_field(self, ls_idx, field):
        self.label_spaces[ls_idx].field = field

    def rescope(self, scoping: dpf.Scoping):  # Used by post.Dataframe
        """Helper function to reproduce functionality of rescope_fc Operator."""
        copy_fc = _MockPropertyFieldsContainer(self, server=None)
        for idx, label_space in enumerate(copy_fc.label_spaces):
            pfield = PropertyField(location=label_space.field.location)
            pfield.data = np.ravel(
                [label_space._field.get_entity_data_by_id(id) for id in scoping.ids]
            )
            pfield.scoping.ids = scoping.ids
            copy_fc._set_field(idx, pfield)
        return copy_fc



"""
vtk_export
==========
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class vtk_export(Operator):
    """Write the input field and fields container into a given vtk path

    Parameters
    ----------
    export_type : int, optional
        Either export volume elements in the mesh
        with their fields with value 0 or
        faces elements in the mesh with their
        fields with value 1 (default is 0)
    file_path : str
        Path with vtk extension were the export
        occurs
    mesh : MeshedRegion, optional
        Necessary if the first field or fields
        container don't have a mesh in their
        support
    fields1 : FieldsContainer or Field
        Fields exported
    fields2 : FieldsContainer or Field
        Fields exported


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.serialization.vtk_export()

    >>> # Make input connections
    >>> my_export_type = int()
    >>> op.inputs.export_type.connect(my_export_type)
    >>> my_file_path = str()
    >>> op.inputs.file_path.connect(my_file_path)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_fields1 = dpf.FieldsContainer()
    >>> op.inputs.fields1.connect(my_fields1)
    >>> my_fields2 = dpf.FieldsContainer()
    >>> op.inputs.fields2.connect(my_fields2)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.serialization.vtk_export(
    ...     export_type=my_export_type,
    ...     file_path=my_file_path,
    ...     mesh=my_mesh,
    ...     fields1=my_fields1,
    ...     fields2=my_fields2,
    ... )

    """

    def __init__(
        self,
        export_type=None,
        file_path=None,
        mesh=None,
        fields1=None,
        fields2=None,
        config=None,
        server=None,
    ):
        super().__init__(name="vtk_export", config=config, server=server)
        self._inputs = InputsVtkExport(self)
        self._outputs = OutputsVtkExport(self)
        if export_type is not None:
            self.inputs.export_type.connect(export_type)
        if file_path is not None:
            self.inputs.file_path.connect(file_path)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)
        if fields1 is not None:
            self.inputs.fields1.connect(fields1)
        if fields2 is not None:
            self.inputs.fields2.connect(fields2)

    @staticmethod
    def _spec():
        description = (
            """Write the input field and fields container into a given vtk path"""
        )
        spec = Specification(
            description=description,
            map_input_pin_spec={
                -1: PinSpecification(
                    name="export_type",
                    type_names=["int32"],
                    optional=True,
                    document="""Either export volume elements in the mesh
        with their fields with value 0 or
        faces elements in the mesh with their
        fields with value 1 (default is 0)""",
                ),
                0: PinSpecification(
                    name="file_path",
                    type_names=["string"],
                    optional=False,
                    document="""Path with vtk extension were the export
        occurs""",
                ),
                1: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=True,
                    document="""Necessary if the first field or fields
        container don't have a mesh in their
        support""",
                ),
                2: PinSpecification(
                    name="fields",
                    type_names=["fields_container", "field"],
                    optional=False,
                    document="""Fields exported""",
                ),
                3: PinSpecification(
                    name="fields",
                    type_names=["fields_container", "field"],
                    optional=False,
                    document="""Fields exported""",
                ),
            },
            map_output_pin_spec={},
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="vtk_export", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsVtkExport
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsVtkExport
        """
        return super().outputs


class InputsVtkExport(_Inputs):
    """Intermediate class used to connect user inputs to
    vtk_export operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.serialization.vtk_export()
    >>> my_export_type = int()
    >>> op.inputs.export_type.connect(my_export_type)
    >>> my_file_path = str()
    >>> op.inputs.file_path.connect(my_file_path)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_fields1 = dpf.FieldsContainer()
    >>> op.inputs.fields1.connect(my_fields1)
    >>> my_fields2 = dpf.FieldsContainer()
    >>> op.inputs.fields2.connect(my_fields2)
    """

    def __init__(self, op: Operator):
        super().__init__(vtk_export._spec().inputs, op)
        self._export_type = Input(vtk_export._spec().input_pin(-1), -1, op, -1)
        self._inputs.append(self._export_type)
        self._file_path = Input(vtk_export._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._file_path)
        self._mesh = Input(vtk_export._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._mesh)
        self._fields1 = Input(vtk_export._spec().input_pin(2), 2, op, 0)
        self._inputs.append(self._fields1)
        self._fields2 = Input(vtk_export._spec().input_pin(3), 3, op, 1)
        self._inputs.append(self._fields2)

    @property
    def export_type(self):
        """Allows to connect export_type input to the operator.

        Either export volume elements in the mesh
        with their fields with value 0 or
        faces elements in the mesh with their
        fields with value 1 (default is 0)

        Parameters
        ----------
        my_export_type : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.vtk_export()
        >>> op.inputs.export_type.connect(my_export_type)
        >>> # or
        >>> op.inputs.export_type(my_export_type)
        """
        return self._export_type

    @property
    def file_path(self):
        """Allows to connect file_path input to the operator.

        Path with vtk extension were the export
        occurs

        Parameters
        ----------
        my_file_path : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.vtk_export()
        >>> op.inputs.file_path.connect(my_file_path)
        >>> # or
        >>> op.inputs.file_path(my_file_path)
        """
        return self._file_path

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Necessary if the first field or fields
        container don't have a mesh in their
        support

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.vtk_export()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh

    @property
    def fields1(self):
        """Allows to connect fields1 input to the operator.

        Fields exported

        Parameters
        ----------
        my_fields1 : FieldsContainer or Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.vtk_export()
        >>> op.inputs.fields1.connect(my_fields1)
        >>> # or
        >>> op.inputs.fields1(my_fields1)
        """
        return self._fields1

    @property
    def fields2(self):
        """Allows to connect fields2 input to the operator.

        Fields exported

        Parameters
        ----------
        my_fields2 : FieldsContainer or Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.vtk_export()
        >>> op.inputs.fields2.connect(my_fields2)
        >>> # or
        >>> op.inputs.fields2(my_fields2)
        """
        return self._fields2


class OutputsVtkExport(_Outputs):
    """Intermediate class used to get outputs from
    vtk_export operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.serialization.vtk_export()
    >>> # Connect inputs : op.inputs. ...
    """

    def __init__(self, op: Operator):
        super().__init__(vtk_export._spec().outputs, op)



"""
identical_property_fields
=========================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class identical_property_fields(Operator):
    """Takes two property fields and compares them.

    Parameters
    ----------
    property_fieldA : MeshedRegion
    property_fieldB : MeshedRegion


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.logic.identical_property_fields()

    >>> # Make input connections
    >>> my_property_fieldA = dpf.MeshedRegion()
    >>> op.inputs.property_fieldA.connect(my_property_fieldA)
    >>> my_property_fieldB = dpf.MeshedRegion()
    >>> op.inputs.property_fieldB.connect(my_property_fieldB)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.logic.identical_property_fields(
    ...     property_fieldA=my_property_fieldA,
    ...     property_fieldB=my_property_fieldB,
    ... )

    >>> # Get output data
    >>> result_are_identical = op.outputs.are_identical()
    >>> result_information = op.outputs.information()
    """

    def __init__(
        self, property_fieldA=None, property_fieldB=None, config=None, server=None
    ):
        super().__init__(name="compare::property_field", config=config, server=server)
        self._inputs = InputsIdenticalPropertyFields(self)
        self._outputs = OutputsIdenticalPropertyFields(self)
        if property_fieldA is not None:
            self.inputs.property_fieldA.connect(property_fieldA)
        if property_fieldB is not None:
            self.inputs.property_fieldB.connect(property_fieldB)

    @staticmethod
    def _spec():
        description = """Takes two property fields and compares them."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="property_fieldA",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="property_fieldB",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="are_identical",
                    type_names=["bool"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="information",
                    type_names=["string"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="compare::property_field", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsIdenticalPropertyFields
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsIdenticalPropertyFields
        """
        return super().outputs


class InputsIdenticalPropertyFields(_Inputs):
    """Intermediate class used to connect user inputs to
    identical_property_fields operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.logic.identical_property_fields()
    >>> my_property_fieldA = dpf.MeshedRegion()
    >>> op.inputs.property_fieldA.connect(my_property_fieldA)
    >>> my_property_fieldB = dpf.MeshedRegion()
    >>> op.inputs.property_fieldB.connect(my_property_fieldB)
    """

    def __init__(self, op: Operator):
        super().__init__(identical_property_fields._spec().inputs, op)
        self._property_fieldA = Input(
            identical_property_fields._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._property_fieldA)
        self._property_fieldB = Input(
            identical_property_fields._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._property_fieldB)

    @property
    def property_fieldA(self):
        """Allows to connect property_fieldA input to the operator.

        Parameters
        ----------
        my_property_fieldA : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.identical_property_fields()
        >>> op.inputs.property_fieldA.connect(my_property_fieldA)
        >>> # or
        >>> op.inputs.property_fieldA(my_property_fieldA)
        """
        return self._property_fieldA

    @property
    def property_fieldB(self):
        """Allows to connect property_fieldB input to the operator.

        Parameters
        ----------
        my_property_fieldB : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.identical_property_fields()
        >>> op.inputs.property_fieldB.connect(my_property_fieldB)
        >>> # or
        >>> op.inputs.property_fieldB(my_property_fieldB)
        """
        return self._property_fieldB


class OutputsIdenticalPropertyFields(_Outputs):
    """Intermediate class used to get outputs from
    identical_property_fields operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.logic.identical_property_fields()
    >>> # Connect inputs : op.inputs. ...
    >>> result_are_identical = op.outputs.are_identical()
    >>> result_information = op.outputs.information()
    """

    def __init__(self, op: Operator):
        super().__init__(identical_property_fields._spec().outputs, op)
        self._are_identical = Output(
            identical_property_fields._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._are_identical)
        self._information = Output(
            identical_property_fields._spec().output_pin(1), 1, op
        )
        self._outputs.append(self._information)

    @property
    def are_identical(self):
        """Allows to get are_identical output of the operator

        Returns
        ----------
        my_are_identical : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.identical_property_fields()
        >>> # Connect inputs : op.inputs. ...
        >>> result_are_identical = op.outputs.are_identical()
        """  # noqa: E501
        return self._are_identical

    @property
    def information(self):
        """Allows to get information output of the operator

        Returns
        ----------
        my_information : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.identical_property_fields()
        >>> # Connect inputs : op.inputs. ...
        >>> result_information = op.outputs.information()
        """  # noqa: E501
        return self._information



"""
html_doc
========
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class html_doc(Operator):
    """Create dpf's html documentation. Only on Windows.

    Parameters
    ----------
    output_path : str, optional
        Default is {working
        directory}/dataprocessingdoc.html
    exposure_level : int, optional
        Generate the documentation depending on
        exposure level : 0 (default) for
        public operators, 1 includes hidden
        operator, 2 includes private
        operator, 3 includes operator without
        specifications.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.html_doc()

    >>> # Make input connections
    >>> my_output_path = str()
    >>> op.inputs.output_path.connect(my_output_path)
    >>> my_exposure_level = int()
    >>> op.inputs.exposure_level.connect(my_exposure_level)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.html_doc(
    ...     output_path=my_output_path,
    ...     exposure_level=my_exposure_level,
    ... )

    """

    def __init__(self, output_path=None, exposure_level=None, config=None, server=None):
        super().__init__(name="html_doc", config=config, server=server)
        self._inputs = InputsHtmlDoc(self)
        self._outputs = OutputsHtmlDoc(self)
        if output_path is not None:
            self.inputs.output_path.connect(output_path)
        if exposure_level is not None:
            self.inputs.exposure_level.connect(exposure_level)

    @staticmethod
    def _spec():
        description = """Create dpf's html documentation. Only on Windows."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="output_path",
                    type_names=["string"],
                    optional=True,
                    document="""Default is {working
        directory}/dataprocessingdoc.html""",
                ),
                1: PinSpecification(
                    name="exposure_level",
                    type_names=["int32"],
                    optional=True,
                    document="""Generate the documentation depending on
        exposure level : 0 (default) for
        public operators, 1 includes hidden
        operator, 2 includes private
        operator, 3 includes operator without
        specifications.""",
                ),
            },
            map_output_pin_spec={},
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="html_doc", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsHtmlDoc
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsHtmlDoc
        """
        return super().outputs


class InputsHtmlDoc(_Inputs):
    """Intermediate class used to connect user inputs to
    html_doc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.html_doc()
    >>> my_output_path = str()
    >>> op.inputs.output_path.connect(my_output_path)
    >>> my_exposure_level = int()
    >>> op.inputs.exposure_level.connect(my_exposure_level)
    """

    def __init__(self, op: Operator):
        super().__init__(html_doc._spec().inputs, op)
        self._output_path = Input(html_doc._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._output_path)
        self._exposure_level = Input(html_doc._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._exposure_level)

    @property
    def output_path(self):
        """Allows to connect output_path input to the operator.

        Default is {working
        directory}/dataprocessingdoc.html

        Parameters
        ----------
        my_output_path : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.html_doc()
        >>> op.inputs.output_path.connect(my_output_path)
        >>> # or
        >>> op.inputs.output_path(my_output_path)
        """
        return self._output_path

    @property
    def exposure_level(self):
        """Allows to connect exposure_level input to the operator.

        Generate the documentation depending on
        exposure level : 0 (default) for
        public operators, 1 includes hidden
        operator, 2 includes private
        operator, 3 includes operator without
        specifications.

        Parameters
        ----------
        my_exposure_level : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.html_doc()
        >>> op.inputs.exposure_level.connect(my_exposure_level)
        >>> # or
        >>> op.inputs.exposure_level(my_exposure_level)
        """
        return self._exposure_level


class OutputsHtmlDoc(_Outputs):
    """Intermediate class used to get outputs from
    html_doc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.html_doc()
    >>> # Connect inputs : op.inputs. ...
    """

    def __init__(self, op: Operator):
        super().__init__(html_doc._spec().outputs, op)



"""
mesh_get_attribute
==================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.outputs import _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class mesh_get_attribute(Operator):
    """Uses the MeshedRegion APIs to return a given attribute of the mesh in
    input.

    Parameters
    ----------
    abstract_meshed_region : MeshedRegion
    property_name : str
        Supported property names are: "connectivity",
        "reverse_connectivity", "mat",
        "faces_nodes_connectivity",
        "elements_faces_connectivity" (or any
        mesh's property field),
        "coordinates", "named_selection",
        "num_named_selections",
        "named_selection_names",
        "named_selection_locations",
        "node_scoping", "element_scoping",
        ace_scoping"...
    property_identifier : int or str, optional
        Can be used to get a property at a given
        index, example: a named selection's
        number or by name, example: a named
        selection's name.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.mesh.mesh_get_attribute()

    >>> # Make input connections
    >>> my_abstract_meshed_region = dpf.MeshedRegion()
    >>> op.inputs.abstract_meshed_region.connect(my_abstract_meshed_region)
    >>> my_property_name = str()
    >>> op.inputs.property_name.connect(my_property_name)
    >>> my_property_identifier = int()
    >>> op.inputs.property_identifier.connect(my_property_identifier)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.mesh.mesh_get_attribute(
    ...     abstract_meshed_region=my_abstract_meshed_region,
    ...     property_name=my_property_name,
    ...     property_identifier=my_property_identifier,
    ... )

    >>> # Get output data
    >>> result_property = op.outputs.property()
    """

    def __init__(
        self,
        abstract_meshed_region=None,
        property_name=None,
        property_identifier=None,
        config=None,
        server=None,
    ):
        super().__init__(name="mesh::get_attribute", config=config, server=server)
        self._inputs = InputsMeshGetAttribute(self)
        self._outputs = OutputsMeshGetAttribute(self)
        if abstract_meshed_region is not None:
            self.inputs.abstract_meshed_region.connect(abstract_meshed_region)
        if property_name is not None:
            self.inputs.property_name.connect(property_name)
        if property_identifier is not None:
            self.inputs.property_identifier.connect(property_identifier)

    @staticmethod
    def _spec():
        description = """Uses the MeshedRegion APIs to return a given attribute of the mesh in
            input."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="abstract_meshed_region",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="property_name",
                    type_names=["string"],
                    optional=False,
                    document="""Supported property names are: "connectivity",
        "reverse_connectivity", "mat",
        "faces_nodes_connectivity",
        "elements_faces_connectivity" (or any
        mesh's property field),
        "coordinates", "named_selection",
        "num_named_selections",
        "named_selection_names",
        "named_selection_locations",
        "node_scoping", "element_scoping",
        ace_scoping"...""",
                ),
                2: PinSpecification(
                    name="property_identifier",
                    type_names=["int32", "string"],
                    optional=True,
                    document="""Can be used to get a property at a given
        index, example: a named selection's
        number or by name, example: a named
        selection's name.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="property",
                    type_names=[
                        "scoping",
                        "field",
                        "property_field",
                        "int32",
                        "string_field",
                    ],
                    optional=False,
                    document="""Returns a property field for properties:
        "connectivity",
        "reverse_connectivity", "mat",
        "faces_nodes_connectivity",
        "elements_faces_connectivity" (or any
        mesh's property field), a field for
        property: "coordinates", a scoping
        for properties:"named_selection",
        "node_scoping", "element_scoping",
        ace_scoping", a string field for
        properties: "named_selection_names",
        "named_selection_locations" and an
        int for property:
        "num_named_selections".""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="mesh::get_attribute", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMeshGetAttribute
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMeshGetAttribute
        """
        return super().outputs


class InputsMeshGetAttribute(_Inputs):
    """Intermediate class used to connect user inputs to
    mesh_get_attribute operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.mesh_get_attribute()
    >>> my_abstract_meshed_region = dpf.MeshedRegion()
    >>> op.inputs.abstract_meshed_region.connect(my_abstract_meshed_region)
    >>> my_property_name = str()
    >>> op.inputs.property_name.connect(my_property_name)
    >>> my_property_identifier = int()
    >>> op.inputs.property_identifier.connect(my_property_identifier)
    """

    def __init__(self, op: Operator):
        super().__init__(mesh_get_attribute._spec().inputs, op)
        self._abstract_meshed_region = Input(
            mesh_get_attribute._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._abstract_meshed_region)
        self._property_name = Input(mesh_get_attribute._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._property_name)
        self._property_identifier = Input(
            mesh_get_attribute._spec().input_pin(2), 2, op, -1
        )
        self._inputs.append(self._property_identifier)

    @property
    def abstract_meshed_region(self):
        """Allows to connect abstract_meshed_region input to the operator.

        Parameters
        ----------
        my_abstract_meshed_region : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_get_attribute()
        >>> op.inputs.abstract_meshed_region.connect(my_abstract_meshed_region)
        >>> # or
        >>> op.inputs.abstract_meshed_region(my_abstract_meshed_region)
        """
        return self._abstract_meshed_region

    @property
    def property_name(self):
        """Allows to connect property_name input to the operator.

        Supported property names are: "connectivity",
        "reverse_connectivity", "mat",
        "faces_nodes_connectivity",
        "elements_faces_connectivity" (or any
        mesh's property field),
        "coordinates", "named_selection",
        "num_named_selections",
        "named_selection_names",
        "named_selection_locations",
        "node_scoping", "element_scoping",
        ace_scoping"...

        Parameters
        ----------
        my_property_name : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_get_attribute()
        >>> op.inputs.property_name.connect(my_property_name)
        >>> # or
        >>> op.inputs.property_name(my_property_name)
        """
        return self._property_name

    @property
    def property_identifier(self):
        """Allows to connect property_identifier input to the operator.

        Can be used to get a property at a given
        index, example: a named selection's
        number or by name, example: a named
        selection's name.

        Parameters
        ----------
        my_property_identifier : int or str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_get_attribute()
        >>> op.inputs.property_identifier.connect(my_property_identifier)
        >>> # or
        >>> op.inputs.property_identifier(my_property_identifier)
        """
        return self._property_identifier


class OutputsMeshGetAttribute(_Outputs):
    """Intermediate class used to get outputs from
    mesh_get_attribute operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.mesh_get_attribute()
    >>> # Connect inputs : op.inputs. ...
    >>> result_property = op.outputs.property()
    """

    def __init__(self, op: Operator):
        super().__init__(mesh_get_attribute._spec().outputs, op)
        self.property_as_scoping = Output(
            _modify_output_spec_with_one_type(
                mesh_get_attribute._spec().output_pin(0), "scoping"
            ),
            0,
            op,
        )
        self._outputs.append(self.property_as_scoping)
        self.property_as_field = Output(
            _modify_output_spec_with_one_type(
                mesh_get_attribute._spec().output_pin(0), "field"
            ),
            0,
            op,
        )
        self._outputs.append(self.property_as_field)
        self.property_as_property_field = Output(
            _modify_output_spec_with_one_type(
                mesh_get_attribute._spec().output_pin(0), "property_field"
            ),
            0,
            op,
        )
        self._outputs.append(self.property_as_property_field)
        self.property_as_int32 = Output(
            _modify_output_spec_with_one_type(
                mesh_get_attribute._spec().output_pin(0), "int32"
            ),
            0,
            op,
        )
        self._outputs.append(self.property_as_int32)
        self.property_as_string_field = Output(
            _modify_output_spec_with_one_type(
                mesh_get_attribute._spec().output_pin(0), "string_field"
            ),
            0,
            op,
        )
        self._outputs.append(self.property_as_string_field)



"""
compute_invariant_terms_rbd
===========================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class compute_invariant_terms_rbd(Operator):
    """Set the required data for the invariant terms computation (reduced
    matrices, lumped mass matrix, modes ...)

    Parameters
    ----------
    rom_matrices : FieldsContainer
        Fieldscontainers containing the reduced
        matrices
    mode_shapes : FieldsContainer
        Fieldscontainers containing the mode shapes,
        which are cst and nor for the cms
        method
    lumped_mass : FieldsContainer
        Fieldscontainers containing the lumped mass
    model_data : FieldsContainer
        Data describing the finite element model
    center_of_mass : FieldsContainer
    inertia_relief : FieldsContainer
        Inertia matrix
    model_size : float
        Model size
    field_coordinates : Field
        Coordinates of all nodes
    nod :


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.result.compute_invariant_terms_rbd()

    >>> # Make input connections
    >>> my_rom_matrices = dpf.FieldsContainer()
    >>> op.inputs.rom_matrices.connect(my_rom_matrices)
    >>> my_mode_shapes = dpf.FieldsContainer()
    >>> op.inputs.mode_shapes.connect(my_mode_shapes)
    >>> my_lumped_mass = dpf.FieldsContainer()
    >>> op.inputs.lumped_mass.connect(my_lumped_mass)
    >>> my_model_data = dpf.FieldsContainer()
    >>> op.inputs.model_data.connect(my_model_data)
    >>> my_center_of_mass = dpf.FieldsContainer()
    >>> op.inputs.center_of_mass.connect(my_center_of_mass)
    >>> my_inertia_relief = dpf.FieldsContainer()
    >>> op.inputs.inertia_relief.connect(my_inertia_relief)
    >>> my_model_size = float()
    >>> op.inputs.model_size.connect(my_model_size)
    >>> my_field_coordinates = dpf.Field()
    >>> op.inputs.field_coordinates.connect(my_field_coordinates)
    >>> my_nod = dpf.()
    >>> op.inputs.nod.connect(my_nod)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.result.compute_invariant_terms_rbd(
    ...     rom_matrices=my_rom_matrices,
    ...     mode_shapes=my_mode_shapes,
    ...     lumped_mass=my_lumped_mass,
    ...     model_data=my_model_data,
    ...     center_of_mass=my_center_of_mass,
    ...     inertia_relief=my_inertia_relief,
    ...     model_size=my_model_size,
    ...     field_coordinates=my_field_coordinates,
    ...     nod=my_nod,
    ... )

    >>> # Get output data
    >>> result_model_data = op.outputs.model_data()
    >>> result_center_of_mass = op.outputs.center_of_mass()
    >>> result_inertia_relief = op.outputs.inertia_relief()
    >>> result_model_size = op.outputs.model_size()
    >>> result_master_node_coordinates = op.outputs.master_node_coordinates()
    >>> result_v_trsf = op.outputs.v_trsf()
    >>> result_k_mat = op.outputs.k_mat()
    >>> result_mass_mat = op.outputs.mass_mat()
    >>> result_c_mat = op.outputs.c_mat()
    >>> result_rhs = op.outputs.rhs()
    >>> result_dn = op.outputs.dn()
    >>> result_dr_cross_n = op.outputs.dr_cross_n()
    >>> result_drn = op.outputs.drn()
    >>> result_dn_cross_n = op.outputs.dn_cross_n()
    >>> result_dnx_y = op.outputs.dnx_y()
    >>> result_dny_y = op.outputs.dny_y()
    >>> result_dnz_y = op.outputs.dnz_y()
    >>> result_dyx_n = op.outputs.dyx_n()
    >>> result_dyy_n = op.outputs.dyy_n()
    >>> result_dyz_n = op.outputs.dyz_n()
    >>> result_dnxn = op.outputs.dnxn()
    >>> result_dnyn = op.outputs.dnyn()
    >>> result_dnzn = op.outputs.dnzn()
    """

    def __init__(
        self,
        rom_matrices=None,
        mode_shapes=None,
        lumped_mass=None,
        model_data=None,
        center_of_mass=None,
        inertia_relief=None,
        model_size=None,
        field_coordinates=None,
        nod=None,
        config=None,
        server=None,
    ):
        super().__init__(
            name="compute_invariant_terms_rbd", config=config, server=server
        )
        self._inputs = InputsComputeInvariantTermsRbd(self)
        self._outputs = OutputsComputeInvariantTermsRbd(self)
        if rom_matrices is not None:
            self.inputs.rom_matrices.connect(rom_matrices)
        if mode_shapes is not None:
            self.inputs.mode_shapes.connect(mode_shapes)
        if lumped_mass is not None:
            self.inputs.lumped_mass.connect(lumped_mass)
        if model_data is not None:
            self.inputs.model_data.connect(model_data)
        if center_of_mass is not None:
            self.inputs.center_of_mass.connect(center_of_mass)
        if inertia_relief is not None:
            self.inputs.inertia_relief.connect(inertia_relief)
        if model_size is not None:
            self.inputs.model_size.connect(model_size)
        if field_coordinates is not None:
            self.inputs.field_coordinates.connect(field_coordinates)
        if nod is not None:
            self.inputs.nod.connect(nod)

    @staticmethod
    def _spec():
        description = """Set the required data for the invariant terms computation (reduced
            matrices, lumped mass matrix, modes ...)"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="rom_matrices",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Fieldscontainers containing the reduced
        matrices""",
                ),
                1: PinSpecification(
                    name="mode_shapes",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Fieldscontainers containing the mode shapes,
        which are cst and nor for the cms
        method""",
                ),
                2: PinSpecification(
                    name="lumped_mass",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Fieldscontainers containing the lumped mass""",
                ),
                3: PinSpecification(
                    name="model_data",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Data describing the finite element model""",
                ),
                4: PinSpecification(
                    name="center_of_mass",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
                5: PinSpecification(
                    name="inertia_relief",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Inertia matrix""",
                ),
                6: PinSpecification(
                    name="model_size",
                    type_names=["double"],
                    optional=False,
                    document="""Model size""",
                ),
                7: PinSpecification(
                    name="field_coordinates",
                    type_names=["field"],
                    optional=False,
                    document="""Coordinates of all nodes""",
                ),
                9: PinSpecification(
                    name="nod",
                    type_names=["vector<int32>"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="model_data",
                    type_names=["property_field"],
                    optional=False,
                    document="""Data describing the finite element model""",
                ),
                1: PinSpecification(
                    name="center_of_mass",
                    type_names=["field"],
                    optional=False,
                    document="""Center of mass of the body""",
                ),
                2: PinSpecification(
                    name="inertia_relief",
                    type_names=["field"],
                    optional=False,
                    document="""Inertia matrix""",
                ),
                3: PinSpecification(
                    name="model_size",
                    type_names=["property_field"],
                    optional=False,
                    document="""""",
                ),
                4: PinSpecification(
                    name="master_node_coordinates",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                5: PinSpecification(
                    name="v_trsf",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""Translational and rotational shape functions""",
                ),
                6: PinSpecification(
                    name="k_mat",
                    type_names=["field"],
                    optional=False,
                    document="""""",
                ),
                7: PinSpecification(
                    name="mass_mat",
                    type_names=["field"],
                    optional=False,
                    document="""""",
                ),
                8: PinSpecification(
                    name="c_mat",
                    type_names=["field"],
                    optional=False,
                    document="""""",
                ),
                9: PinSpecification(
                    name="rhs",
                    type_names=["field"],
                    optional=False,
                    document="""""",
                ),
                10: PinSpecification(
                    name="dn",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                11: PinSpecification(
                    name="dr_cross_n",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                12: PinSpecification(
                    name="drn",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                13: PinSpecification(
                    name="dn_cross_n",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                14: PinSpecification(
                    name="dnx_y",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                15: PinSpecification(
                    name="dny_y",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                16: PinSpecification(
                    name="dnz_y",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                17: PinSpecification(
                    name="dyx_n",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                18: PinSpecification(
                    name="dyy_n",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                19: PinSpecification(
                    name="dyz_n",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                20: PinSpecification(
                    name="dnxn",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                21: PinSpecification(
                    name="dnyn",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                22: PinSpecification(
                    name="dnzn",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(
            name="compute_invariant_terms_rbd", server=server
        )

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsComputeInvariantTermsRbd
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsComputeInvariantTermsRbd
        """
        return super().outputs


class InputsComputeInvariantTermsRbd(_Inputs):
    """Intermediate class used to connect user inputs to
    compute_invariant_terms_rbd operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.compute_invariant_terms_rbd()
    >>> my_rom_matrices = dpf.FieldsContainer()
    >>> op.inputs.rom_matrices.connect(my_rom_matrices)
    >>> my_mode_shapes = dpf.FieldsContainer()
    >>> op.inputs.mode_shapes.connect(my_mode_shapes)
    >>> my_lumped_mass = dpf.FieldsContainer()
    >>> op.inputs.lumped_mass.connect(my_lumped_mass)
    >>> my_model_data = dpf.FieldsContainer()
    >>> op.inputs.model_data.connect(my_model_data)
    >>> my_center_of_mass = dpf.FieldsContainer()
    >>> op.inputs.center_of_mass.connect(my_center_of_mass)
    >>> my_inertia_relief = dpf.FieldsContainer()
    >>> op.inputs.inertia_relief.connect(my_inertia_relief)
    >>> my_model_size = float()
    >>> op.inputs.model_size.connect(my_model_size)
    >>> my_field_coordinates = dpf.Field()
    >>> op.inputs.field_coordinates.connect(my_field_coordinates)
    >>> my_nod = dpf.()
    >>> op.inputs.nod.connect(my_nod)
    """

    def __init__(self, op: Operator):
        super().__init__(compute_invariant_terms_rbd._spec().inputs, op)
        self._rom_matrices = Input(
            compute_invariant_terms_rbd._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._rom_matrices)
        self._mode_shapes = Input(
            compute_invariant_terms_rbd._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._mode_shapes)
        self._lumped_mass = Input(
            compute_invariant_terms_rbd._spec().input_pin(2), 2, op, -1
        )
        self._inputs.append(self._lumped_mass)
        self._model_data = Input(
            compute_invariant_terms_rbd._spec().input_pin(3), 3, op, -1
        )
        self._inputs.append(self._model_data)
        self._center_of_mass = Input(
            compute_invariant_terms_rbd._spec().input_pin(4), 4, op, -1
        )
        self._inputs.append(self._center_of_mass)
        self._inertia_relief = Input(
            compute_invariant_terms_rbd._spec().input_pin(5), 5, op, -1
        )
        self._inputs.append(self._inertia_relief)
        self._model_size = Input(
            compute_invariant_terms_rbd._spec().input_pin(6), 6, op, -1
        )
        self._inputs.append(self._model_size)
        self._field_coordinates = Input(
            compute_invariant_terms_rbd._spec().input_pin(7), 7, op, -1
        )
        self._inputs.append(self._field_coordinates)
        self._nod = Input(compute_invariant_terms_rbd._spec().input_pin(9), 9, op, -1)
        self._inputs.append(self._nod)

    @property
    def rom_matrices(self):
        """Allows to connect rom_matrices input to the operator.

        Fieldscontainers containing the reduced
        matrices

        Parameters
        ----------
        my_rom_matrices : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> op.inputs.rom_matrices.connect(my_rom_matrices)
        >>> # or
        >>> op.inputs.rom_matrices(my_rom_matrices)
        """
        return self._rom_matrices

    @property
    def mode_shapes(self):
        """Allows to connect mode_shapes input to the operator.

        Fieldscontainers containing the mode shapes,
        which are cst and nor for the cms
        method

        Parameters
        ----------
        my_mode_shapes : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> op.inputs.mode_shapes.connect(my_mode_shapes)
        >>> # or
        >>> op.inputs.mode_shapes(my_mode_shapes)
        """
        return self._mode_shapes

    @property
    def lumped_mass(self):
        """Allows to connect lumped_mass input to the operator.

        Fieldscontainers containing the lumped mass

        Parameters
        ----------
        my_lumped_mass : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> op.inputs.lumped_mass.connect(my_lumped_mass)
        >>> # or
        >>> op.inputs.lumped_mass(my_lumped_mass)
        """
        return self._lumped_mass

    @property
    def model_data(self):
        """Allows to connect model_data input to the operator.

        Data describing the finite element model

        Parameters
        ----------
        my_model_data : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> op.inputs.model_data.connect(my_model_data)
        >>> # or
        >>> op.inputs.model_data(my_model_data)
        """
        return self._model_data

    @property
    def center_of_mass(self):
        """Allows to connect center_of_mass input to the operator.

        Parameters
        ----------
        my_center_of_mass : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> op.inputs.center_of_mass.connect(my_center_of_mass)
        >>> # or
        >>> op.inputs.center_of_mass(my_center_of_mass)
        """
        return self._center_of_mass

    @property
    def inertia_relief(self):
        """Allows to connect inertia_relief input to the operator.

        Inertia matrix

        Parameters
        ----------
        my_inertia_relief : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> op.inputs.inertia_relief.connect(my_inertia_relief)
        >>> # or
        >>> op.inputs.inertia_relief(my_inertia_relief)
        """
        return self._inertia_relief

    @property
    def model_size(self):
        """Allows to connect model_size input to the operator.

        Model size

        Parameters
        ----------
        my_model_size : float

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> op.inputs.model_size.connect(my_model_size)
        >>> # or
        >>> op.inputs.model_size(my_model_size)
        """
        return self._model_size

    @property
    def field_coordinates(self):
        """Allows to connect field_coordinates input to the operator.

        Coordinates of all nodes

        Parameters
        ----------
        my_field_coordinates : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> op.inputs.field_coordinates.connect(my_field_coordinates)
        >>> # or
        >>> op.inputs.field_coordinates(my_field_coordinates)
        """
        return self._field_coordinates

    @property
    def nod(self):
        """Allows to connect nod input to the operator.

        Parameters
        ----------
        my_nod :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> op.inputs.nod.connect(my_nod)
        >>> # or
        >>> op.inputs.nod(my_nod)
        """
        return self._nod


class OutputsComputeInvariantTermsRbd(_Outputs):
    """Intermediate class used to get outputs from
    compute_invariant_terms_rbd operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.compute_invariant_terms_rbd()
    >>> # Connect inputs : op.inputs. ...
    >>> result_model_data = op.outputs.model_data()
    >>> result_center_of_mass = op.outputs.center_of_mass()
    >>> result_inertia_relief = op.outputs.inertia_relief()
    >>> result_model_size = op.outputs.model_size()
    >>> result_master_node_coordinates = op.outputs.master_node_coordinates()
    >>> result_v_trsf = op.outputs.v_trsf()
    >>> result_k_mat = op.outputs.k_mat()
    >>> result_mass_mat = op.outputs.mass_mat()
    >>> result_c_mat = op.outputs.c_mat()
    >>> result_rhs = op.outputs.rhs()
    >>> result_dn = op.outputs.dn()
    >>> result_dr_cross_n = op.outputs.dr_cross_n()
    >>> result_drn = op.outputs.drn()
    >>> result_dn_cross_n = op.outputs.dn_cross_n()
    >>> result_dnx_y = op.outputs.dnx_y()
    >>> result_dny_y = op.outputs.dny_y()
    >>> result_dnz_y = op.outputs.dnz_y()
    >>> result_dyx_n = op.outputs.dyx_n()
    >>> result_dyy_n = op.outputs.dyy_n()
    >>> result_dyz_n = op.outputs.dyz_n()
    >>> result_dnxn = op.outputs.dnxn()
    >>> result_dnyn = op.outputs.dnyn()
    >>> result_dnzn = op.outputs.dnzn()
    """

    def __init__(self, op: Operator):
        super().__init__(compute_invariant_terms_rbd._spec().outputs, op)
        self._model_data = Output(
            compute_invariant_terms_rbd._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._model_data)
        self._center_of_mass = Output(
            compute_invariant_terms_rbd._spec().output_pin(1), 1, op
        )
        self._outputs.append(self._center_of_mass)
        self._inertia_relief = Output(
            compute_invariant_terms_rbd._spec().output_pin(2), 2, op
        )
        self._outputs.append(self._inertia_relief)
        self._model_size = Output(
            compute_invariant_terms_rbd._spec().output_pin(3), 3, op
        )
        self._outputs.append(self._model_size)
        self._master_node_coordinates = Output(
            compute_invariant_terms_rbd._spec().output_pin(4), 4, op
        )
        self._outputs.append(self._master_node_coordinates)
        self._v_trsf = Output(compute_invariant_terms_rbd._spec().output_pin(5), 5, op)
        self._outputs.append(self._v_trsf)
        self._k_mat = Output(compute_invariant_terms_rbd._spec().output_pin(6), 6, op)
        self._outputs.append(self._k_mat)
        self._mass_mat = Output(
            compute_invariant_terms_rbd._spec().output_pin(7), 7, op
        )
        self._outputs.append(self._mass_mat)
        self._c_mat = Output(compute_invariant_terms_rbd._spec().output_pin(8), 8, op)
        self._outputs.append(self._c_mat)
        self._rhs = Output(compute_invariant_terms_rbd._spec().output_pin(9), 9, op)
        self._outputs.append(self._rhs)
        self._dn = Output(compute_invariant_terms_rbd._spec().output_pin(10), 10, op)
        self._outputs.append(self._dn)
        self._dr_cross_n = Output(
            compute_invariant_terms_rbd._spec().output_pin(11), 11, op
        )
        self._outputs.append(self._dr_cross_n)
        self._drn = Output(compute_invariant_terms_rbd._spec().output_pin(12), 12, op)
        self._outputs.append(self._drn)
        self._dn_cross_n = Output(
            compute_invariant_terms_rbd._spec().output_pin(13), 13, op
        )
        self._outputs.append(self._dn_cross_n)
        self._dnx_y = Output(compute_invariant_terms_rbd._spec().output_pin(14), 14, op)
        self._outputs.append(self._dnx_y)
        self._dny_y = Output(compute_invariant_terms_rbd._spec().output_pin(15), 15, op)
        self._outputs.append(self._dny_y)
        self._dnz_y = Output(compute_invariant_terms_rbd._spec().output_pin(16), 16, op)
        self._outputs.append(self._dnz_y)
        self._dyx_n = Output(compute_invariant_terms_rbd._spec().output_pin(17), 17, op)
        self._outputs.append(self._dyx_n)
        self._dyy_n = Output(compute_invariant_terms_rbd._spec().output_pin(18), 18, op)
        self._outputs.append(self._dyy_n)
        self._dyz_n = Output(compute_invariant_terms_rbd._spec().output_pin(19), 19, op)
        self._outputs.append(self._dyz_n)
        self._dnxn = Output(compute_invariant_terms_rbd._spec().output_pin(20), 20, op)
        self._outputs.append(self._dnxn)
        self._dnyn = Output(compute_invariant_terms_rbd._spec().output_pin(21), 21, op)
        self._outputs.append(self._dnyn)
        self._dnzn = Output(compute_invariant_terms_rbd._spec().output_pin(22), 22, op)
        self._outputs.append(self._dnzn)

    @property
    def model_data(self):
        """Allows to get model_data output of the operator

        Returns
        ----------
        my_model_data : PropertyField

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_model_data = op.outputs.model_data()
        """  # noqa: E501
        return self._model_data

    @property
    def center_of_mass(self):
        """Allows to get center_of_mass output of the operator

        Returns
        ----------
        my_center_of_mass : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_center_of_mass = op.outputs.center_of_mass()
        """  # noqa: E501
        return self._center_of_mass

    @property
    def inertia_relief(self):
        """Allows to get inertia_relief output of the operator

        Returns
        ----------
        my_inertia_relief : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_inertia_relief = op.outputs.inertia_relief()
        """  # noqa: E501
        return self._inertia_relief

    @property
    def model_size(self):
        """Allows to get model_size output of the operator

        Returns
        ----------
        my_model_size : PropertyField

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_model_size = op.outputs.model_size()
        """  # noqa: E501
        return self._model_size

    @property
    def master_node_coordinates(self):
        """Allows to get master_node_coordinates output of the operator

        Returns
        ----------
        my_master_node_coordinates :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_master_node_coordinates = op.outputs.master_node_coordinates()
        """  # noqa: E501
        return self._master_node_coordinates

    @property
    def v_trsf(self):
        """Allows to get v_trsf output of the operator

        Returns
        ----------
        my_v_trsf :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_v_trsf = op.outputs.v_trsf()
        """  # noqa: E501
        return self._v_trsf

    @property
    def k_mat(self):
        """Allows to get k_mat output of the operator

        Returns
        ----------
        my_k_mat : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_k_mat = op.outputs.k_mat()
        """  # noqa: E501
        return self._k_mat

    @property
    def mass_mat(self):
        """Allows to get mass_mat output of the operator

        Returns
        ----------
        my_mass_mat : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_mass_mat = op.outputs.mass_mat()
        """  # noqa: E501
        return self._mass_mat

    @property
    def c_mat(self):
        """Allows to get c_mat output of the operator

        Returns
        ----------
        my_c_mat : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_c_mat = op.outputs.c_mat()
        """  # noqa: E501
        return self._c_mat

    @property
    def rhs(self):
        """Allows to get rhs output of the operator

        Returns
        ----------
        my_rhs : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_rhs = op.outputs.rhs()
        """  # noqa: E501
        return self._rhs

    @property
    def dn(self):
        """Allows to get dn output of the operator

        Returns
        ----------
        my_dn :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_dn = op.outputs.dn()
        """  # noqa: E501
        return self._dn

    @property
    def dr_cross_n(self):
        """Allows to get dr_cross_n output of the operator

        Returns
        ----------
        my_dr_cross_n :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_dr_cross_n = op.outputs.dr_cross_n()
        """  # noqa: E501
        return self._dr_cross_n

    @property
    def drn(self):
        """Allows to get drn output of the operator

        Returns
        ----------
        my_drn :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_drn = op.outputs.drn()
        """  # noqa: E501
        return self._drn

    @property
    def dn_cross_n(self):
        """Allows to get dn_cross_n output of the operator

        Returns
        ----------
        my_dn_cross_n :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_dn_cross_n = op.outputs.dn_cross_n()
        """  # noqa: E501
        return self._dn_cross_n

    @property
    def dnx_y(self):
        """Allows to get dnx_y output of the operator

        Returns
        ----------
        my_dnx_y :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_dnx_y = op.outputs.dnx_y()
        """  # noqa: E501
        return self._dnx_y

    @property
    def dny_y(self):
        """Allows to get dny_y output of the operator

        Returns
        ----------
        my_dny_y :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_dny_y = op.outputs.dny_y()
        """  # noqa: E501
        return self._dny_y

    @property
    def dnz_y(self):
        """Allows to get dnz_y output of the operator

        Returns
        ----------
        my_dnz_y :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_dnz_y = op.outputs.dnz_y()
        """  # noqa: E501
        return self._dnz_y

    @property
    def dyx_n(self):
        """Allows to get dyx_n output of the operator

        Returns
        ----------
        my_dyx_n :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_dyx_n = op.outputs.dyx_n()
        """  # noqa: E501
        return self._dyx_n

    @property
    def dyy_n(self):
        """Allows to get dyy_n output of the operator

        Returns
        ----------
        my_dyy_n :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_dyy_n = op.outputs.dyy_n()
        """  # noqa: E501
        return self._dyy_n

    @property
    def dyz_n(self):
        """Allows to get dyz_n output of the operator

        Returns
        ----------
        my_dyz_n :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_dyz_n = op.outputs.dyz_n()
        """  # noqa: E501
        return self._dyz_n

    @property
    def dnxn(self):
        """Allows to get dnxn output of the operator

        Returns
        ----------
        my_dnxn :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_dnxn = op.outputs.dnxn()
        """  # noqa: E501
        return self._dnxn

    @property
    def dnyn(self):
        """Allows to get dnyn output of the operator

        Returns
        ----------
        my_dnyn :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_dnyn = op.outputs.dnyn()
        """  # noqa: E501
        return self._dnyn

    @property
    def dnzn(self):
        """Allows to get dnzn output of the operator

        Returns
        ----------
        my_dnzn :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_rbd()
        >>> # Connect inputs : op.inputs. ...
        >>> result_dnzn = op.outputs.dnzn()
        """  # noqa: E501
        return self._dnzn



"""
stl_export
==========
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class stl_export(Operator):
    """export a mesh into a stl file.

    Parameters
    ----------
    mesh : MeshedRegion
    file_path : str


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.mesh.stl_export()

    >>> # Make input connections
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_file_path = str()
    >>> op.inputs.file_path.connect(my_file_path)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.mesh.stl_export(
    ...     mesh=my_mesh,
    ...     file_path=my_file_path,
    ... )

    >>> # Get output data
    >>> result_data_sources = op.outputs.data_sources()
    """

    def __init__(self, mesh=None, file_path=None, config=None, server=None):
        super().__init__(name="stl_export", config=config, server=server)
        self._inputs = InputsStlExport(self)
        self._outputs = OutputsStlExport(self)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)
        if file_path is not None:
            self.inputs.file_path.connect(file_path)

    @staticmethod
    def _spec():
        description = """export a mesh into a stl file."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="file_path",
                    type_names=["string"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="stl_export", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsStlExport
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsStlExport
        """
        return super().outputs


class InputsStlExport(_Inputs):
    """Intermediate class used to connect user inputs to
    stl_export operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.stl_export()
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_file_path = str()
    >>> op.inputs.file_path.connect(my_file_path)
    """

    def __init__(self, op: Operator):
        super().__init__(stl_export._spec().inputs, op)
        self._mesh = Input(stl_export._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._mesh)
        self._file_path = Input(stl_export._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._file_path)

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.stl_export()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh

    @property
    def file_path(self):
        """Allows to connect file_path input to the operator.

        Parameters
        ----------
        my_file_path : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.stl_export()
        >>> op.inputs.file_path.connect(my_file_path)
        >>> # or
        >>> op.inputs.file_path(my_file_path)
        """
        return self._file_path


class OutputsStlExport(_Outputs):
    """Intermediate class used to get outputs from
    stl_export operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.stl_export()
    >>> # Connect inputs : op.inputs. ...
    >>> result_data_sources = op.outputs.data_sources()
    """

    def __init__(self, op: Operator):
        super().__init__(stl_export._spec().outputs, op)
        self._data_sources = Output(stl_export._spec().output_pin(0), 0, op)
        self._outputs.append(self._data_sources)

    @property
    def data_sources(self):
        """Allows to get data_sources output of the operator

        Returns
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.stl_export()
        >>> # Connect inputs : op.inputs. ...
        >>> result_data_sources = op.outputs.data_sources()
        """  # noqa: E501
        return self._data_sources
